/* 	Server.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 23-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * TODO berekening totalen voor groepen gaat niet goed		
 * TODO connection could get lost (time-out)
 */
package org.brains2b.coffee;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.brains2b.coffee.decode.Decoder;
import org.brains2b.coffee.decode.DecoderFactory;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.GroupList;
import org.brains2b.coffee.message.InfoMessage;
import org.brains2b.coffee.message.MultiPartGroup;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.PostArticle;
import org.brains2b.coffee.message.local.ErrorMessage;
import org.brains2b.coffee.message.local.ServerDescriptor;
import org.brains2b.coffee.monitor.ProgressMonitor;
import org.brains2b.coffee.nntp.NNTPConnector;
import org.brains2b.log.Logger;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class NewsServer implements TicketHandler {
    
    private static final int[] EVENTS = new int[] {
        CoffeeEvent.GET_ARTICLE,
        CoffeeEvent.GET_GROUPLIST,
        CoffeeEvent.GET_ARTICLES,
        CoffeeEvent.GET_ATTACHMENT,
        CoffeeEvent.SAVE_ATTACHMENT,
        CoffeeEvent.SAVE_POST,
        CoffeeEvent.POST_ARTICLE
    };
                                                
    
    private ServerDescriptor m_server;
    private NNTPConnector[] m_nntps;
    private String m_lastGroup;
    private int m_maxCon;
    private List m_monitors;
    /**
     * 
     */
    
    public NewsServer(ServerDescriptor server) {
        super();
        m_server=server;
        m_maxCon = m_server.getMaxConnections();
        m_nntps = new NNTPConnector[m_maxCon];
    }
    
    public NewsServer(String address, int port,String path) {
        this(new ServerDescriptor(address,port,path));
    }
    
    public NewsServer(String address, int port, File path) {
        this(new ServerDescriptor(address,port,path.getAbsolutePath()));
    }
    
    public NNTPMessage handleEvent(CoffeeEvent event) throws CoffeeSpill {
        NNTPMessage message=null;
        switch (event.getID()) {
            case CoffeeEvent.GET_GROUPLIST: //8000:
                message=getGroupList();
                break;
            case CoffeeEvent.GET_ARTICLES: //8011
                message=getNewArticles(new MultiPartGroup((NewsGroup) event.getArgument()));
                break;
            case CoffeeEvent.GET_ARTICLE: //8020
            case CoffeeEvent.SAVE_POST:
                Article a=(Article) event.getArgument();
                if (a.getNumber()!=0) {
                    message=getArticle(a.getNewsGroup(),a.getNumber());
                } else {
                    message=getArticle(a.getNewsGroup(),a.getId());
                }
                if (message instanceof Article) {
                    ((Article) message).setTotalParts(a.getTotalParts());
                    ((Article) message).setParts(a.getParts());
                
                	if (event.getID()==CoffeeEvent.SAVE_POST && ((Article) message).getTotalParts()>1) {
                        for(int i=0;i<((Article) message).getParts().size();i++) {
                            String id =(String) ((Article) message).getParts().get(i);
                            if (id!=null) {
                                NNTPMessage part=getArticle(((Article) message).getNewsGroup(),id);
                                TicketBroker.getBroker().broker(
                                	new CoffeeEvent((CoffeeExchange) event.getSource(),CoffeeEvent.SAVE_POST,part)
                                );
                            }
                        }
                    }
                } else if (message instanceof InfoMessage && message.getType()==423) {
                	a.setAvailable(false);
                }
                break;
            case CoffeeEvent.GET_ATTACHMENT: //8021
            case CoffeeEvent.SAVE_ATTACHMENT: //8022
                Article article=(Article) event.getArgument();
                if (article.getBody()==null) { 
                    NNTPMessage m=getArticle(article.getNewsGroup(),article.getId());
                    if (m instanceof Article) {
                        ((Article) m).setTotalParts(article.getTotalParts());
                        ((Article) m).setParts(article.getParts());
                        article= (Article) m;
                    } else {
                        article=null;
                    }
                    
                }
                if (article!=null) {
                    try {
                    Decoder decoder=DecoderFactory.createDecoder(article);
                    if (article.getTotalParts()>1) {
                        for(int i=0;i<article.getParts().size();i++) {
                            String id =(String) article.getParts().get(i);
                            if (id!=null) {
                                NNTPMessage part=getArticle(article.getNewsGroup(),id);
                                if (part!=null) {
                                    decoder.decodePiece(((Article) part).getBody().getContent());
                                }
                            }
                        }
                    }
                    article.setFile(decoder.getFileMessage());
                    message=article;
                    } catch (Exception ex) {
                        throw new CoffeeSpill("Cannot decode message",ex);
                    }
                }
                
                break;
            case CoffeeEvent.POST_ARTICLE:
                message = post((Article) event.getArgument());
        }
        if (message instanceof InfoMessage && message.getType()==412) { //Not in a newsgroup
        	m_lastGroup=null;
        	handleEvent(event);
        }
        return message;
    }

    private NNTPConnector getConnection() throws Exception {
        NNTPConnector nntp=null;
        synchronized(m_nntps) {
        while (true) {
            for (int i=0;i<getMaxConnection();i++) {
                if (m_nntps[i]==null) {
                    m_nntps[i]=new NNTPConnector(m_server.getAddress(),m_server.getPort(), m_server.getUsername(), m_server.getPassword());
                    try {
                        NNTPMessage msg= m_nntps[i].connect();
                        if (msg.getType()==400) {
                            setMaxConnection(getMaxConnection()-1);
                            m_nntps[i]=null;
                            break;
                        }
                    } catch (Exception ex) {
                        Logger.printException(ex);
                        m_nntps[i]=null;
                    }
                    nntp= m_nntps[i];
                    break;
                } else if (m_nntps[i].isAvailable()) {
                    nntp= m_nntps[i];
                    break;
                }
            }
            if (nntp!=null) {
        		nntp.setAvailable(false);
                for (int i=0;i<m_monitors.size();i++) {
                	nntp.getParser().addProgressMonitor((ProgressMonitor) m_monitors.get(i));
            	}
        		break;
            } else {
                try { 
                	m_nntps.wait(); 
                	Logger.println("released");
                } catch (InterruptedException iex) {}
            }
        }
        }
        return nntp;
    }
    
    private void releaseConnection(NNTPConnector nntp) {
        synchronized (m_nntps) {
			if (nntp!=null) {
	            for (int i=0;i<m_monitors.size();i++) {
	                nntp.getParser().removeProgressMonitor((ProgressMonitor) m_monitors.get(i));
	            }
	            nntp.setAvailable(true);
	            m_nntps.notifyAll();
	        }
        }
    }
    
    public GroupList getGroupList() {
    	NNTPConnector nntp = null;
        GroupList list=null;
        try {
            nntp=getConnection();
        	list=(GroupList) nntp.invoke("LIST","active");
        	//fc.save(list,"groups");
            
        } catch (Exception ex) {
        	Logger.printException(ex);
        } finally {
        	releaseConnection(nntp);
        }
        return list;
    }
    
    public NNTPMessage getNewArticles(NewsGroup group) {
        NNTPConnector nntp=null;
        try {
            nntp=getConnection();
        	NNTPMessage message=nntp.invoke("GROUP",group.getName());
	        if (message instanceof InfoMessage) {
	        	return message;
	        }
        	NewsGroup grp=(NewsGroup) message;
            if (group.getLast()>=grp.getLast()) {
                return group;
            }
	        int retrieve=grp.getFirst();
           
	        if (grp.getFirst()<group.getLast()) {
	          retrieve=group.getLast()+1;
            }
            if (grp.getLast()-retrieve>50000) { 
            	group.getArticleList().clear();
            	retrieve=grp.getLast()-50000; 
            }
            group.setFirst(Math.max(grp.getLast()-50000,group.getFirst()));
            group.setLast(grp.getLast());
            
            nntp.invokeOn(group,"XOVER",retrieve+"-");
			return group;
			
        } catch (Exception ex) {
        	Logger.printException(ex);
           	return new ErrorMessage(ex);
        } finally {
        	releaseConnection(nntp);
        }
        
    }
    
    public NNTPMessage getArticle(int number) {
        return getArticle(null,number);
    }
    public NNTPMessage getArticle(String groupName,String messageId) {
            return getArticle(groupName,0,messageId);
    }
    
    public NNTPMessage getArticle(String groupName,int type, String value) {
        
        NNTPConnector nntp=null;
        if (m_lastGroup==null || !m_lastGroup.equals(groupName)) {
            try {
                nntp=getConnection();
           		nntp.invoke("GROUP",groupName);
           		m_lastGroup=groupName;
            } catch (Exception ex) {
            	
            } finally {
            	releaseConnection(nntp);
            }
        }
        
        if (type==0) {
            value="<"+value+">";
        }
        
        try {
            if (nntp==null) {
                nntp=getConnection();
            }
            NNTPMessage message=nntp.invoke("ARTICLE",value);
            return message;
        } catch (Exception ex) {
        	return null;    
        } finally {
            releaseConnection(nntp);
        }
        
    }
    
    public NNTPMessage getArticle(String groupName,int number) {
       return getArticle(groupName,1,String.valueOf(number));
        
    }
    
    public NNTPMessage post(Article a) {
        NNTPConnector nntp = null;
        try {
            nntp=getConnection();
            return nntp.post(PostArticle.create(a));
           
        } catch (Exception ex) {
        	ex.printStackTrace();
            return null;
        } finally {
            releaseConnection(nntp);
        }
        
    }
    
    public String getAddress() {
        return m_server.getAddress();
    }
    
    public int getMaxConnection() {
        return m_maxCon;
    }
    public void setMaxConnection(int maxConnection) {
        m_maxCon=maxConnection;
    }
    
    public void close() throws Exception {
        for (int i=0;i<m_nntps.length;i++) {
            if( m_nntps[i]!=null) {
                m_nntps[i].close();
            }
        }
        
    }
        
    public boolean isAvailable() {
    	for (int i=0;i<getMaxConnection();i++) {
            if (m_nntps[i]==null || m_nntps[i].isAvailable()) {
                return true;
            }
        }
    	return false;
    }

    public void addProgressMonitor(ProgressMonitor monitor) {
        if (m_monitors==null) {
            m_monitors=new ArrayList();
        }
        m_monitors.add(monitor);
    }
    
    public void open() {
        
    }

    public void removeProgressMonitor(ProgressMonitor monitor) {
        if (m_monitors!=null) {
            m_monitors.remove(monitor);
        }
    }
    
    public int[] getHandledEvents() {
        return EVENTS;
    }

	public ServerDescriptor getDescriptor() {
		return m_server;
	}
    
}
