/* 	UUDecoder.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 5-aug-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * 26-08-2007	dennis@brains2b.org		Implemented Encoder
 * TODO Are we allowed to use sun.misc.UUEncoder here?		
 */
package org.brains2b.coffee.decode;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStream;

import org.brains2b.coffee.message.local.FileMessage;

import sun.misc.UUEncoder;

/** Decoder and Encoder for UUEncoding
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 1.00 [26-08-2007]
 */
public class UUDecoder implements Decoder, Encoder{
	
    private sun.misc.UUDecoder m_decoder;
    private sun.misc.UUEncoder m_encoder;
    
    private StringBuffer m_buffer=new StringBuffer();
    private File m_tempFile;
    private OutputStream m_os;
    private int m_part=0;
    
    public UUDecoder() {
        m_decoder=new sun.misc.UUDecoder();
    }
    
    public void decodePiece(String s) throws Exception {
        int idx=s.indexOf("begin");
        if (m_buffer.length()==0 && idx>-1) {
            s=s.substring(idx);
            m_part=0;
        } else {
        	m_part++;
        }
        
        s=unixfyString(s);
        while (idx==-1 && s.startsWith("\n")) {
    		s=s.substring(1);
    	}
       	if (s.indexOf("\nend")!=-1) {
       	    s=fixUudecode(s);
       	} 
       	if (m_part>1) {
       		if (m_tempFile ==null) {
       			m_tempFile = File.createTempFile("uue", ".tmp");
       			m_os = new FileOutputStream(m_tempFile);
       			m_os.write(m_buffer.toString().getBytes());
       			m_buffer.delete(0, m_buffer.length());
       		} 
       		m_os.write(s.getBytes());
       	} else {
       		m_buffer.append(s);
       	}
    }
    public FileMessage getFileMessage() throws Exception {
    	byte[] b =null;
    	if (m_tempFile==null) {
        	b =m_decoder.decodeBuffer(m_buffer.toString());
    	} else {
    		m_os.close();
    		m_os = null;
    		b = m_decoder.decodeBuffer(new FileInputStream(m_tempFile));
    		m_tempFile =null;
    	}
        m_buffer=null;
        FileMessage fm=new FileMessage();
        fm.setFile(new File(m_decoder.bufferName));
        fm.setData(b);
        return fm;
    }
    
    /**
     * replace <code>\\r\\n</code> with <code>\\n</code>
     * and replace <code>\\n\\n</code> with <code>\\n</code>
     * for the Decoder
     * @param content String, the content to replace the end of lines for
     * @return String, the content with the end-of-lines replaced
     */
    private final String unixfyString(String content) {
        content= content.replaceAll("\r\n", "\n");
        return content.replaceAll("\n\n", "\n");
    }
    
    /**
     * fix the end of the supplied encoded content
     * so we can supply it to the {@link sun.misc.UUDecoder}
     * which expects <code>\\n \\nend</code>
     * @param content String
     * @return
     */
    private String fixUudecode(String content) {
        int idx=content.indexOf("end");
        int length=idx-1;
        boolean first=true;
        boolean removed=false;
        for (int i=length;i>0;i--) {
            if (content.charAt(i)=='\n') {
                idx--;
                if (first) {
                	first=false;
                } else {
                    break;
                }
            } else if (!removed && (content.charAt(i)=='`' || content.charAt(i)==' ')) {
                idx--;
                removed=true;
            }  else {
                break;
            }
        }
        StringBuffer sb=new StringBuffer(idx+6);
        sb.append(content.substring(0,idx));
        sb.append("`\n \nend");
        return sb.toString();
    }

    /**
     * {@inheritDoc}
     * <p>size is adjusted down from the supplied size to
     * be a multiple of 45, so UUEncode will encode
     * whole lines for multi-part message</p>
     */
	public String encode(File f, int part, int size) {
		try {
			size = size-size%45;
			
			m_encoder=new UUEncoder(f.getName());
			BufferedInputStream bis = new BufferedInputStream(new FileInputStream(f));
			int offset = (part-1)*size;
			int len    = (int) part*size>f.length()?(int) f.length()-offset:size;
			byte[] b = new byte[len];
			bis.skip(offset);
			bis.read(b);
			
			String content = m_encoder.encode(b);
			content = fixUuencode(content, part, part*size>f.length());
			bis.close();
			return content;
		} catch (Exception ex) {
			ex.printStackTrace();
			return null;		
		} finally {
			
		}
		
	}
	
	/**
	 * fix the UUEncoded content from what {@link UUEncoder} 
	 * supplied based on the part that is being encoded
	 * @param content String
	 * @param part int, the part that is begin encoded starting at 1
	 * @param last boolean, is this the last part?
	 * @return String, that can be validly supplied as an NNTPMessage
	 */
	private String fixUuencode(String content, int part, boolean last) {
		StringBuffer sb = new StringBuffer(content);
		if (part>1) {
			int idx = sb.indexOf("\n");
			if (idx>0) sb.delete(0, idx); 
		} 
		sb.insert(0, "\n");
		int idx = sb.indexOf(" \nend");
		if (idx>0) sb.delete(idx, sb.length()-1);
		if (last)  sb.append("`\nend");
		return sb.toString();
	}
}
