/* 	FileContentHandler.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 23-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.file;

import java.util.ArrayList;
import java.util.List;

import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.monitor.ProgressMonitor;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public abstract class FileContentHandler implements ContentHandler, ProgressMonitor {

    
    private List m_monitors;

	public abstract NNTPMessage getMessage(); 
    
    // Not used generally but overwrite if necessary
	public void endPrefixMapping(String prefix) throws SAXException {
	}

	public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
	}

	public void processingInstruction(String target, String data) throws SAXException {
	}

	public void setDocumentLocator(Locator locator) {
	}

	public void skippedEntity(String name) throws SAXException {
	}

	public void startPrefixMapping(String prefix, String uri) throws SAXException {
	}

	public void addProgressMonitor(ProgressMonitor monitor) {
	    if (m_monitors==null) {
	        m_monitors=new ArrayList();
	    }
	    m_monitors.add(monitor);
	}

	/* (non-Javadoc)
	 * @see org.brains2b.coffee.monitor.ProgressMonitor#advance(int, int, int)
	 */
	public void advance(int start, int value, int end) {
	    if (m_monitors!=null) {
	        for (int i=0;i<m_monitors.size();i++) {
	            ((ProgressMonitor) m_monitors.get(i)).advance(start,value,end);
	        }
	    }
	}

	public void removeProgressMonitor(ProgressMonitor monitor) {
	    if (m_monitors!=null) {
	        m_monitors.remove(monitor);
	        if (m_monitors.isEmpty()) m_monitors=null;
	    }
	}
}
