/* 	GroupListHandler.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 23-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * TODO size should be stored with header for tracking purposes		
 */
package org.brains2b.coffee.file;

import org.brains2b.coffee.message.GroupList;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.filter.MessageFilter;
import org.brains2b.coffee.message.local.Subscription;
import org.brains2b.util.DateHelper;
import org.brains2b.util.XmlHelper;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class GroupListHandler extends FileContentHandler {

    private int m_total=0;
    private GroupList m_list;
    /**
     * 
     */
    public GroupListHandler() {
        super();
    }

    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#endDocument()
     */
    public void endDocument() throws SAXException {
       
    }

    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#startDocument()
     */
    public void startDocument() throws SAXException {
        m_list=new GroupList();
       
    }

   
    public void characters(char[] ch, int start, int length)
            throws SAXException {
    }

    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
     */
    public void endElement(String namespaceURI, String localName, String qName)
            throws SAXException {
        int size=m_list.getSize();
        if (m_total<100 || size%(int) (m_total*.01)==0) {
            advance(0,size,m_total);
        }
        
    }

    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public void startElement(String namespaceURI, String localName,
            String qName, Attributes atts) throws SAXException {
       
        if (qName.equals("newsgroups")) {
            try {
                m_total=Integer.parseInt(atts.getValue("size"));
            } catch (NumberFormatException nfex) {
            }
        }
        if (qName.equals("newsgroup")) {
            m_list.addGroup(createNewsGroup(atts));
           
        } else if (qName.equals("subscription")) {
            m_list.addGroup(createSubscription(atts));
        }

    }
    
    public NNTPMessage getMessage() {
        return m_list;
    }
    
    
    private NewsGroup createNewsGroup(Attributes atts) {
        NewsGroup group=new NewsGroup();
        fillGroup(atts,group);
        return group;
    }

    private Subscription createSubscription(Attributes atts) {
        Subscription group=new Subscription();
        fillGroup(atts,group);
        group.setServer(atts.getValue("server"));
        return group;
    }
    
    private void fillGroup(Attributes atts, NewsGroup group) {
        group.setName(XmlHelper.decode(atts.getValue("name")));
        group.setFirst(Integer.parseInt(atts.getValue("first")));
        group.setLast(Integer.parseInt(atts.getValue("last")));
        group.setAllowPost("true".equals(atts.getValue("allowPost")));
        group.setLastUpdated(DateHelper.toDate(atts.getValue("lastUpdated")));
    }
    
    public void setFilter(MessageFilter filter) {
        
    }
}
