/* 	CfMessageView.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 15-jan-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.gui;

import java.awt.BorderLayout;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetAdapter;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.StringTokenizer;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JMenu;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;

import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.gui.binary.CfBinarySend;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.ArticleBody;
import org.brains2b.coffee.message.ArticleHeader;
import org.brains2b.coffee.message.InfoMessage;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.local.FileListMessage;
import org.brains2b.coffee.message.local.FileMessage;
import org.brains2b.coffee.message.local.Identity;
import org.brains2b.coffee.plugin.CoffeeExchangeView;
import org.brains2b.swingext.dialog.DialogHelper;
import org.brains2b.swingext.form.Form;
import org.brains2b.swingext.form.FormAction;
import org.brains2b.util.StringHelper;

public class CfEditMessage extends JPanel implements CoffeeExchangeView, Form, ItemListener, ActionListener {

	class FileDropTarget extends DropTargetAdapter {

		
		public void drop(DropTargetDropEvent dtde) {
			Transferable t= dtde.getTransferable();
			DataFlavor uriListFlavor = null;
			try {
				uriListFlavor = new DataFlavor("text/uri-list;class=java.lang.String");
			} catch (ClassNotFoundException cnfex) {}
			try {
				List files = null;
				if (t.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
					dtde.acceptDrop(DnDConstants.ACTION_LINK);
					files = (List) t.getTransferData(DataFlavor.javaFileListFlavor);
				} else if (t.isDataFlavorSupported(uriListFlavor)) {
					dtde.acceptDrop(DnDConstants.ACTION_LINK);
					files = makeFiles((String) t.getTransferData(uriListFlavor));
				}
				if (files!=null) {
					FileMessage fm = m_article==null?null:m_article.getFile();
					if (fm==null) {
						fm = new FileListMessage();
						((FileListMessage) fm).setFiles(files);
					} else {
						((FileListMessage) fm).setFiles(files);
					}
					if (m_article==null) { m_article = new Article();}
					m_article.setFile(fm);
					m_headerPane.setCurrentMessage(m_article);	
				}
				dtde.dropComplete(true);

			} catch (Exception iex) {
				iex.printStackTrace();
				dtde.dropComplete(false);
				return;
			}
		}
		
		private List makeFiles(String s) {
			if (StringHelper.isEmpty(s)) return null;
			StringTokenizer t = new StringTokenizer(s,"\r\n");
			List l = new ArrayList(t.countTokens());
			while(t.hasMoreTokens()) {
				try {
					URL url = new URL(t.nextToken());
					File f = new File(url.getFile());
					l.add(f);
				} catch (Exception  usex) {} 
			}
			return l;
		}
		
	}
	private Article m_article;
    private CfEditHeaderPane m_headerPane;
    private CfEditPane m_editorPane;
    private JComboBox m_tagLines;
    private Random m_rnd = new Random();
    
    public CfEditMessage() {
        super();
        init();
    }
    
    public void activate() {
        fireCoffeeEvent(m_headerPane, CoffeeEvent.GET_LIST, "identity");

    }

    public void deactivate() {
        m_headerPane.deactivate();
        
    }

    public void fireCoffeeEvent(CoffeeExchange source, int action,
            Object argument) {
        TicketBroker.getBroker().broker(new CoffeeEvent(source,action,argument));

    }

    public JMenu[] getMenus() {
        return null;
    }

    public void setCurrentMessage(NNTPMessage message) {
        if (message instanceof NewsGroup) {
            m_article = new Article();
            m_article.setHeader(new ArticleHeader());
            m_article.getHeader().addHeaderValue("Newsgroups", ((NewsGroup) message).getName());
        } else if (message instanceof Article) {
            if (isEdit()) {
                m_article = (Article) message;
            } else {
                m_article = new Article();
                m_article.setHeader(getFollowupHeader((Article) message));
                m_article.setBody(getQuoteBody((Article) message ));
            }
        } else if (message instanceof InfoMessage) {
            if (message.getType()==240) {
                SwingUtilities.getWindowAncestor(this).dispose();
            } else {
                JOptionPane.showMessageDialog(this, ((InfoMessage)message).getResponse(),"Error",JOptionPane.ERROR_MESSAGE);
            }
        }
        
        if (m_article!=null) {
            m_editorPane.setBody(m_article.getBody());
            m_headerPane.setHeader(m_article.getHeader());
        }

    }

    
    private void init() {
        setLayout(new BorderLayout());
        setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        m_headerPane = new CfEditHeaderPane();
        m_headerPane.addItemListener(this);
        m_headerPane.addActionListener(this);
        m_editorPane = new CfEditPane();
        FileDropTarget fdt = new FileDropTarget();
        new DropTarget(this,fdt);
        add(m_headerPane,BorderLayout.NORTH);
        add(new JScrollPane(m_editorPane),BorderLayout.CENTER);
        JPanel bottom = new JPanel(new BorderLayout());
        m_tagLines=new JComboBox();
        bottom.add(m_tagLines,BorderLayout.NORTH);
        FormAction fa = new FormAction(this, FormAction.BTN_ALL);
        fa.getButton(FormAction.BTN_OK).setText("Send");
        fa.getButton(FormAction.BTN_APPLY).setText("Save");
        bottom.add(fa,BorderLayout.SOUTH);
        add(bottom,BorderLayout.SOUTH);
    }

    public void performApply() {
    	if (m_article==null) {
            m_article = new Article();
        }
        m_article.setHeader(m_headerPane.getHeader());
        m_article.setBody(m_editorPane.getBody());
        if (m_tagLines.getSelectedItem()!=null) {
        	m_article.getBody().addLine("\r\n\r\n" + (String) m_tagLines.getSelectedItem());
        }
        fireCoffeeEvent(this, CoffeeEvent.SAVE_POST,m_article);
    }

    public void performCancel() {
        SwingUtilities.getWindowAncestor(this).dispose();
        
    }

    public void performOk() {
        if (m_article==null) {
            m_article = new Article();
        }
        m_article.setHeader(m_headerPane.getHeader());
        m_article.setBody(m_editorPane.getBody());
        if (m_tagLines.getSelectedItem()!=null) {
        	m_article.getBody().addLine("\r\n\r\n" + (String) m_tagLines.getSelectedItem());
        }
        
        fireCoffeeEvent(this, m_article.getFile()!=null?CoffeeEvent.ENCODE_ARTICLE:CoffeeEvent.POST_ARTICLE, m_article);
        SwingUtilities.getWindowAncestor(this).dispose();
    }

    public void performUndo() {
       m_editorPane.setBody(m_article.getBody());
       m_headerPane.setHeader(m_article.getHeader());
        
    }
    
    public boolean isEdit() { return false; }
    
    private ArticleHeader getFollowupHeader(Article a) {
        ArticleHeader ah =new ArticleHeader();
        ah.addHeaderValue("Newsgroups",(String) a.getHeader().getHeaderValue("Newsgroups"));
        String ref=a.getHeader().getHeaderValue("References");
        if (ref==null) {
            ah.addHeaderValue("References","<" + a.getId() + ">");
        } else {
            ah.addHeaderValue("References", ref+ " <" + a.getId() + ">");
        }
        String subject = a.getHeader().getSubject();
        if (!subject.toUpperCase().startsWith("RE:")) {
            ah.setSubject("Re: " + subject);
        } else {
            ah.setSubject(subject);
        }
        return ah;
    }
    
    private ArticleBody getQuoteBody(Article a) {
        if (a.getBody()==null) return null;
        StringTokenizer st = new StringTokenizer(a.getBody().getContent(),"\r\n");
        ArticleBody ab = new ArticleBody();
        while (st.hasMoreTokens()) {
            ab.addLine(">"+st.nextToken() + "\r\n");
        }
        return ab;
    }

	public void itemStateChanged(ItemEvent e) {
		if (e.getStateChange()==ItemEvent.SELECTED) {
			m_tagLines.removeAllItems();
			if (e.getItem() instanceof Identity) {
				Identity id = (Identity) e.getItem();
				if (id.getTagLines()!=null) {
					for (Iterator it = id.getTagLines().iterator();it.hasNext();) {
						m_tagLines.addItem(it.next());
					}
					int idx = id.getTagline();
					if (id.getTagline()==-1) {
						m_rnd.nextInt(id.getTagLines().size());
					}
					m_tagLines.setSelectedIndex(idx);
				}	
			} else {
				m_tagLines.setEnabled(false);
			}
		}
	}

	public void actionPerformed(ActionEvent e) {
		if ("Message.attach".equals(e.getActionCommand())) {
			if (m_article==null) {
            	m_article = new Article();
        	}
       		m_article.setHeader(m_headerPane.getHeader());
			CfBinarySend binSend = new CfBinarySend();
			binSend.activate();
			binSend.setCurrentMessage(m_article);
			DialogHelper.showDialog(this, binSend, "Configure attachments");
			m_headerPane.setCurrentMessage(m_article);
		}
		
	}
    
}
