/* 	CfFilterPane.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 19-aug-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import org.brains2b.coffee.message.ArticleHeader;
import org.brains2b.coffee.message.GroupList;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.filter.FilterList;
import org.brains2b.coffee.message.filter.MessageFilter;
import org.brains2b.coffee.plugin.CoffeeExchangeView;
import org.brains2b.swingext.action.ExtActionMap;
import org.brains2b.swingext.form.Form;
import org.brains2b.swingext.form.FormAction;
import org.brains2b.swingext.gui.IconFactory;
import org.brains2b.swingext.gui.ScrollablePanel;
import org.brains2b.swingext.gui.SelectablePanel;
import org.brains2b.util.StringHelper;

public class CfFilterPane extends JPanel implements Form,ActionListener {
    
    private class ActionHandler implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            List filters=m_list.getFilterFor((String) ((JComboBox) e.getSource()).getSelectedItem());
            m_filterPane.removeAll();
            if (filters!=null) { 
                for (int i=0;i<filters.size();i++) {
                    m_filterPane.add(new CfFilterRow((MessageFilter) filters.get(i)));
                }
             } else {
                m_filterPane.add(new CfFilterRow(new MessageFilter()));
            }
            m_filterPane.getParent().validate();
        }
    }
    
    private class CfFilterRow extends JPanel implements SelectablePanel{
        private JComboBox m_field;
        private JComboBox m_actions;
        private JTextField m_value;
        private JCheckBox m_kill;
        private boolean m_selected;
        
        public CfFilterRow() {
            super();
            init();
        }
        
         public CfFilterRow(MessageFilter mf) {
            super();
            init();
            setMessageFilter(mf);
        }
        
        private void init() {
            m_field=new JComboBox(ArticleHeader.TYPES);
            add(m_field);
            m_actions=new JComboBox(MessageFilter.OPERATIONS);
            add(m_actions);
            m_value=new JTextField();
            m_value.setColumns(20);
            add(m_value);
            m_kill=new JCheckBox("kill");
            add(m_kill);
            Dimension d=new Dimension(512,40);
            setPreferredSize(d);
            setMinimumSize(d);
            setMaximumSize(new Dimension(1024,40));
            setBorder(BorderFactory.createEtchedBorder());
        }
        
        public void setMessageFilter(MessageFilter mf) {
            m_field.setSelectedItem(mf.getKey());
            m_actions.setSelectedIndex(mf.getOperator());
            m_value.setText(mf.getValue()!=null?mf.getValue().toString():null);
            m_kill.setSelected(mf.isKill());
        }
        
        public MessageFilter getMessageFilter() {
            if (StringHelper.isEmpty(m_value.getText())) {
            	return null;
            }
            MessageFilter mf = new MessageFilter();
            mf.setKey((String) m_field.getSelectedItem());
            mf.setOperator(m_actions.getSelectedIndex());
            mf.setValue(m_value.getText());
            mf.setKill(m_kill.isSelected());
            mf.setNot(true);
            return mf;
        }

		public boolean isSelected() {
			return m_selected;
		}

		public void setSelected(boolean b) {
			m_selected = b;			
		}
        
    }
    
    private NNTPMessage m_message;
    private ScrollablePanel m_filterPane;
    private JComboBox m_groupBox;
    private FilterList m_list;
    private CoffeeExchangeView m_view;
    
    public CfFilterPane(CoffeeExchangeView view) {
        super();
        m_view=view;
        init();
    }
    
    
    private void init() {
        setLayout(new BorderLayout());
        m_filterPane=new ScrollablePanel(ScrollablePanel.VERTICAL);
        JScrollPane scroll= new JScrollPane(m_filterPane);
        scroll.setPreferredSize(new Dimension(512+32,40*6));
        ExtActionMap map=new ExtActionMap(this);
        map.addAction("ExcludeFilter.add",null,"add a filter",null,"Ins",'i');
        map.addAction("ExcludeFilter.delete",null,"delete a filter",null,"Del",'d');
        setActionMap(map);
        JPanel pnl=new JPanel();
        pnl.setLayout(new BoxLayout(pnl,BoxLayout.Y_AXIS));
        JButton btn=new JButton(getActionMap().get("ExcludeFilter.add"));
        btn.setIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Plus16.gif"));
        btn.setPreferredSize(new Dimension(24,24));
        pnl.add(btn);
        btn=new JButton(getActionMap().get("ExcludeFilter.delete"));
        btn.setIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Minus16.gif"));
        btn.setPreferredSize(new Dimension(24,24));
        pnl.add(btn);
        add(pnl,BorderLayout.EAST);
        add(scroll,BorderLayout.CENTER);
        FormAction fa=new FormAction(this,FormAction.BTN_OK+FormAction.BTN_CANCEL);
        add(fa,BorderLayout.SOUTH);
    }
    
    public void setFilters(FilterList list) {
        m_list=list;
    }
    
    public void setFilterContext(NNTPMessage message) {
        m_message=message;
      
        List filters=null;
        if (message instanceof NewsGroup) {
            filters=m_list.getFilterFor(((NewsGroup)message).getName());
            addMessageHeader(((NewsGroup)message).getName());
        } else {
            filters=m_list.getFilterFor(GroupList.class.getName());
            addGroupHeader();
        }
        if (filters!=null) { 
            for (int i=0;i<filters.size();i++) {
                m_filterPane.add(new CfFilterRow((MessageFilter) filters.get(i)));
            }
        } else {
            m_filterPane.add(new CfFilterRow(new MessageFilter()));
        }
    }


    public void performApply() {
        throw new NullPointerException("No// TODO Auto-generated method stubt defined");
    }


    public void performCancel() {
       SwingUtilities.windowForComponent(this).dispose();
        
    }


    public void performOk() {
        List filters = new ArrayList();
    	for (int i=0;i<m_filterPane.getComponentCount();i++) {
            CfFilterRow row= (CfFilterRow) m_filterPane.getComponent(i);
            MessageFilter mf = row.getMessageFilter();
            if (mf!=null)
            filters.add(mf);
        }
        if (!filters.isEmpty() && m_message instanceof NewsGroup) {
            m_list.setFilterFor((String) m_groupBox.getSelectedItem(),filters);
        } else if (!filters.isEmpty()){
            m_list.setFilterFor(m_message.getClass().getName(),filters);
        }
        
        m_view.setCurrentMessage(m_message);	
        
        
       SwingUtilities.windowForComponent(this).dispose();
    }

    public void performUndo() {
        throw new NullPointerException("Not defined");
    }


    public void actionPerformed(ActionEvent e) {
    	String actionCommand = e.getActionCommand();
    	if ("ExcludeFilter.delete".equals(actionCommand)) {
            int idx = m_filterPane.getSelectedIndex();
            if (idx!=-1) {
            	m_filterPane.remove(idx);
            }
            m_filterPane.getParent().validate();
        } else if ("ExcludeFilter.add".equals(actionCommand)) { 
            m_filterPane.add(new CfFilterRow(new MessageFilter()));
            m_filterPane.getParent().validate();
        }
        
    }
    
    private void addMessageHeader(String name) {
        JPanel panel=new JPanel(new FlowLayout(FlowLayout.LEFT));
        panel.add(new JLabel("Exclusion filters for: "));
        List list=m_list.getFilterNames();
        if (list == null) {
        	list = new ArrayList();
        }
        int idx=0;
        if (!list.contains(name)) {
            list.add(0, name);
        } else {
            idx=list.indexOf(name);
        }
        m_groupBox=new JComboBox(new Vector(list));
        m_groupBox.setSelectedIndex(idx);
        m_groupBox.addActionListener(new ActionHandler());
        panel.add(m_groupBox);
        add(panel,BorderLayout.NORTH);
    }
    
    private void addGroupHeader() {
        JPanel panel=new JPanel(new FlowLayout(FlowLayout.LEFT));
        panel.add(new JLabel("Exclude groups matching the following:"));
        add(panel,BorderLayout.NORTH);
    }
}
