/* 	CfMessagePane.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 25-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.JTableHeader;

import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.gui.table.ArticleColumnModel;
import org.brains2b.coffee.gui.table.ArticleNode;
import org.brains2b.coffee.gui.table.ArticleTableCellRenderer;
import org.brains2b.coffee.gui.table.ArticleTreeCellRenderer;
import org.brains2b.coffee.gui.table.ArticleTreeEditor;
import org.brains2b.coffee.gui.table.ArticleTreeTableModel;
import org.brains2b.coffee.gui.table.TreeMaker;
import org.brains2b.coffee.gui.table.TreeSelectionAdapter;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.InfoMessage;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.filter.FilterList;
import org.brains2b.coffee.message.filter.FilterNewsGroup;
import org.brains2b.coffee.plugin.CoffeeExchangeView;
import org.brains2b.swingext.action.ExtActionMap;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class CfMessagePane extends JPanel implements CoffeeExchangeView,ActionListener {
	
    class MouseHandler extends MouseAdapter {
        /* (non-Javadoc)
         * @see java.awt.event.MouseAdapter#mouseClicked(java.awt.event.MouseEvent)
         */
        public void mouseClicked(MouseEvent e) {
            if (e.getSource() instanceof JTableHeader) {
            	JTableHeader header=(JTableHeader) e.getSource();
            	int idx=header.columnAtPoint(e.getPoint());
            	m_articleModel.sort(idx,false);
            }
           	super.mouseClicked(e);
        }
        
        /* (non-Javadoc)
         * @see java.awt.event.MouseAdapter#mousePressed(java.awt.event.MouseEvent)
         */
        public void mousePressed(MouseEvent e) {
            if (e.isPopupTrigger()) {
            	popup(e);
            }
            super.mousePressed(e);
        }
        
        /* (non-Javadoc)
         * @see java.awt.event.MouseAdapter#mouseReleased(java.awt.event.MouseEvent)
         */
        public void mouseReleased(MouseEvent e) {
            if (e.isPopupTrigger()) {
            	popup(e);
            }
            super.mouseReleased(e);
        }
        
        void popup(MouseEvent e) {
            if (e.getSource() instanceof JTable) {
                JTable t= (JTable)e.getSource();
                int[] rows=t.getSelectedRows();
                
                if (rows!=null && rows.length>1) {
                    m_selected.clear();
                	for (int i=0;i<rows.length;i++) {
	             		m_selected.add(m_articleModel.getArticle(rows[i]));       
	                }
                } else {
                	int row = t.rowAtPoint(e.getPoint());
                	t.getSelectionModel().setSelectionInterval(row, row);
                }
                getPopMenu().show(t,e.getPoint().x,e.getPoint().y);
            }
        }
    }
   
    class SelectionHandler implements ListSelectionListener {
        /* (non-Javadoc)
         * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
         */
        public void valueChanged(ListSelectionEvent e) {
            if (!e.getValueIsAdjusting()) {
                ListSelectionModel model = (ListSelectionModel) e.getSource();
                if (model.getMinSelectionIndex()==model.getMaxSelectionIndex()) {
                    m_selected.clear();
                    Article a=m_articleModel.getArticle(model.getMinSelectionIndex());
                    if (a!=null) {
                        a.setNewsGroup(m_newsGroup.getName());
                        handleGetArticle(a);
                        m_selected.add(a);
                        getActionMap().get("Message.followup").setEnabled(true);
                    } else {
                        getActionMap().get("Message.followup").setEnabled(false);
                    }
                }
            }

        }
    }
    ArticleTreeTableModel m_articleModel;
    private CfPreviewPane m_preview;
    private JSplitPane m_split;
   	private NewsGroup m_newsGroup;
   	private JPopupMenu m_popMenu;
   	private Vector m_selected=new Vector();
    private CfFilterDelegate m_filter;
    private CfStatsDelegate m_stats;
    public CfMessagePane() {
        super();
        init();
    }
    
    private void init() {
       	setLayout(new BorderLayout());
        m_filter=new CfFilterDelegate(this);
        m_stats=new CfStatsDelegate(this);
        m_split=new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    	
    	ExtActionMap map=new ExtActionMap(this);
    	map.addAction("save.attach","Save attachements",null,null,"ctrl shift s",'s');
    	map.addAction("save.message","Save message",null,null,"ctrl s",'s');
        map.addAction("newsgroup.refresh","Refresh",null,"/org/brains2b/coffee/resource/Refresh16.gif","ctrl r",'r');
        map.addAction("Message.new","Post message",null,null,"ctrl n",'n');
        map.addAction("Message.followup","Post follow-up",null,null,"ctrl f",'f');
       	map.addAction("Filter.poster","Exclude poster",null,null,"ctrl k",'0');
       	map.addAction("Filter.posterFromAll","Exclude posterFromAll",null,null,"ctrl shift k",'0');
       	map.addAction("Filter.message","Exclude ...",null,null,"ctrl shift e",'0');
        
       	map.get("Message.followup").setEnabled(false);
        setActionMap(map);
    	JTable table=new JTable() {
            public int getEditingRow() {
                return getColumnClass(editingColumn)==ArticleNode.class? -1 : editingRow;   
            }
        };
        m_articleModel=new ArticleTreeTableModel();
        ArticleTreeCellRenderer articleRender = new ArticleTreeCellRenderer(m_articleModel,table);
        m_articleModel.setTree(articleRender);
        table.setDefaultEditor(ArticleNode.class, new ArticleTreeEditor(articleRender));
        table.setDefaultRenderer(ArticleNode.class, articleRender);
    	table.setDefaultRenderer(String.class, new ArticleTableCellRenderer());
        TreeSelectionAdapter selModel = new TreeSelectionAdapter(table.getSelectionModel());
       	articleRender.setSelectionModel(selModel);
        table.getSelectionModel().addListSelectionListener(new SelectionHandler());
        table.setModel(m_articleModel);
       	table.setColumnModel(new ArticleColumnModel());
        table.setShowGrid(false);
        table.setIntercellSpacing(new Dimension(0,0));
        articleRender.setRowHeight(table.getRowHeight());
        JPanel listPane=new JPanel(new BorderLayout());
        JScrollPane tablePane=new JScrollPane(table);
        listPane.add(tablePane,BorderLayout.CENTER);
        listPane.add(m_stats.addStats(NewsGroup.class),BorderLayout.SOUTH);
       	MouseHandler mh=new MouseHandler();
       	table.getTableHeader().addMouseListener(mh);
       	table.addMouseListener(mh);
       	m_preview=new CfPreviewPane(this);
       	JScrollPane previewPane=new JScrollPane();
       	previewPane.setViewportView(m_preview);
       	
       	m_split.setTopComponent(listPane);
       	m_split.setBottomComponent(previewPane);
        JPanel top = new JPanel();
        add(top,BorderLayout.NORTH);
        map.get("newsgroup.refresh").setEnabled(false);
        JButton btn= new JButton(map.get("newsgroup.refresh"));
        btn.setText(null);
        btn.setPreferredSize(new Dimension(24,24));
        top.add(btn);
        top.add(m_filter.addFilter(8017,"Subject",30));
       	add(m_split,BorderLayout.CENTER);

    }
    
    /* (non-Javadoc)
     * @see java.awt.Container#validate()
     */
    
    public void fireCoffeeEvent(CoffeeExchange source, int action, Object argument) {
        TicketBroker.getBroker().broker(new CoffeeEvent(source,action,argument));
    }
    public void setCurrentMessage(NNTPMessage message) {
        if (message instanceof NewsGroup) {
            /*if (m_newsGroup==null || !m_newsGroup.getName().equals(((NewsGroup) message).getName())) {
               m_filter.clearFilter();
            }*/
            m_newsGroup=(NewsGroup) message;
            FilterNewsGroup fng = new FilterNewsGroup(m_newsGroup,FilterList.getInstance().getAllFilterFor(m_newsGroup.getId()));
            ArticleNode node=TreeMaker.order(fng.getArticleList());
            m_articleModel.setData(node);
            m_filter.setCurrentMessage(message);
            m_stats.setCurrentMessage(message);
            getActionMap().get("newsgroup.refresh").setEnabled(true);
        } else if (message instanceof Article) {
            if (m_selected.size()<=1) {
            	Article a=(Article) message;
                if (!m_selected.isEmpty() && a.getId().equals(((Article) m_selected.get(0)).getId())) {
                   ((Article) m_selected.get(0)).setNew(false);
            	   m_preview.setArticle(a);
                }
            }
        } else if (message instanceof InfoMessage && message.getType()==423) {
        	
        }
    }
    private void handleGetArticle(Article a) {
        a.setNewsGroup(m_newsGroup.getName());
        fireCoffeeEvent(this,CoffeeEvent.GET_ARTICLE,a);
    }
    /* (non-Javadoc)
     * @see org.brains2b.coffee.plugin.CoffeeExchangeView#getMenus()
     */
    public JMenu[] getMenus() {
        JMenu menu[]= new JMenu[1];
        menu[0]= new JMenu("Message");
        menu[0].add(getActionMap().get("Message.new"));
        menu[0].add(getActionMap().get("Message.followup"));
        return menu;
    }
    
    /* (non-Javadoc)
     * @see org.brains2b.coffee.plugin.CoffeeExchangeView#activate()
     */
    public void activate() {
        m_split.setDividerLocation(0.40d);
    }
    
    /* (non-Javadoc)
     * @see org.brains2b.coffee.plugin.CoffeeExchangeView#deactivate()
     */
    public void deactivate() {
        
    }
    
    private JPopupMenu getPopMenu() {
        if (m_popMenu==null) {
        	m_popMenu=new JPopupMenu("artice list");
			m_popMenu.add(new JMenuItem(getActionMap().get("Message.followup")));
        	m_popMenu.add(new JMenuItem(getActionMap().get("save.attach")));
        	m_popMenu.add(new JMenuItem(getActionMap().get("save.message")));
        	m_popMenu.addSeparator();
        	m_popMenu.add(new JMenuItem(getActionMap().get("Filter.poster")));
        	m_popMenu.add(new JMenuItem(getActionMap().get("Filter.posterFromAll")));
        	m_popMenu.add(new JMenuItem(getActionMap().get("Filter.message")));
        }
        return m_popMenu;
    }
    
    public void actionPerformed(ActionEvent e) {
    	String actionCommand = e.getActionCommand();
    	if (actionCommand.equals("save.attach")) {
            if (!m_selected.isEmpty()) {
                for (int i=0;i<m_selected.size();i++) {
                	fireCoffeeEvent(this,CoffeeEvent.SAVE_ATTACHMENT,m_selected.get(i));
                }	
            }
    	} else if (actionCommand.equals("save.message")) {
    		if (!m_selected.isEmpty()) {
                for (int i=0;i<m_selected.size();i++) {
                	fireCoffeeEvent(this,CoffeeEvent.SAVE_POST,m_selected.get(i));
                }	
            }
    	} else if (actionCommand.startsWith("Filter")) {
    		m_filter.actionPerformed(new ActionEvent(m_selected,e.getID(),actionCommand));
    	} else if (actionCommand.equals("newsgroup.refresh")) {
            fireCoffeeEvent(this, CoffeeEvent.GET_ARTICLES, m_newsGroup);
        } else if (actionCommand.equals("Message.new")) {
            JFrame frm = (JFrame) SwingUtilities.getWindowAncestor(this);
            JDialog dlg=new JDialog(frm,false);
            CfEditMessage msg=new CfEditMessage();
            dlg.setContentPane(msg);
            dlg.setLocationRelativeTo(this);
            dlg.pack();
            dlg.show();
            msg.activate();
            msg.setCurrentMessage(m_newsGroup);
        } else if (actionCommand.equals("Message.followup")) {
            JFrame frm = (JFrame) SwingUtilities.getWindowAncestor(this);
            JDialog dlg=new JDialog(frm,false);
            CfEditMessage msg=new CfEditMessage();
            dlg.setContentPane(msg);
            dlg.setLocationRelativeTo(this);
            dlg.pack();
            dlg.show();
            msg.activate();
            msg.setCurrentMessage(m_preview.getArticle());
        }

    }
    
}
