/* 	CfPreviewPane.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 28-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * TODO Implement Mime-type lookup		
 */
package org.brains2b.coffee.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.StringTokenizer;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTextPane;
import javax.swing.Scrollable;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;

import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.local.FileMessage;
import org.brains2b.coffee.plugin.CoffeeExchangeView;
import org.brains2b.log.Logger;
import org.brains2b.swingext.action.ExtActionMap;
import org.brains2b.swingext.gui.IconFactory;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class CfPreviewPane extends JPanel implements Scrollable {
    
    private class ActionHandler implements ActionListener {
        /* (non-Javadoc)
         * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
         */
        public void actionPerformed(ActionEvent e) {
            if ("Preview.default".equals(e.getActionCommand()) && "nzb".equals(getType(m_fileMessage.getFile().getPath()))) {
                m_fileMessage.setMimeType("text/xml.nzb");
            	m_view.fireCoffeeEvent(m_view,CoffeeEvent.USE,m_fileMessage);
        	} else if ("Preview.saveAttach".equals(e.getActionCommand())) {
        		m_view.fireCoffeeEvent(m_view, CoffeeEvent.SAVE, m_fileMessage);
        	}
        }
    }
    
    class MouseHandler extends MouseAdapter {
    	private boolean m_on;
    	
    	public void mousePressed(MouseEvent e) {
    		popupMenu(e);
    		super.mousePressed(e);
    	}
    	
    	public void mouseReleased(MouseEvent e) {
    		popupMenu(e);
    		super.mouseReleased(e);
    	}
    	
    	private void popupMenu(MouseEvent e) {
    		if (m_on && e.isPopupTrigger()) {
    			
    			JPopupMenu p = new JPopupMenu();
    			p.add(getActionMap().get("Preview.saveAttach"));
				p.show(m_preview, e.getX(), e.getY());	   				
    			
    		}
    	}
    	
    	public void setPopupOn(boolean b) {
    		m_on = b;
    	}
    }
    private CoffeeExchangeView m_view;
    private FileMessage m_fileMessage;
    private CfHeaderPane m_header;
    private Article m_article;
    private JTextPane m_preview;
    private MouseHandler m_mousie;
    
    public CfPreviewPane(CoffeeExchangeView view) {
         super();
         m_view=view;
         init();
     }
     
     public void setArticle(Article a) {
         m_article = a;
         m_header.setHeader(a);
         if (a.getFile()!=null) {
          	setFile(a.getFile());   
         } else {
         	m_mousie.setPopupOn(false);
        	setBody(a);
         	
         }
     }
     
     public void setFile(FileMessage fm) {
         //TODO Want to see the text surrounding the File
         //TODO Should be mime-type
         m_fileMessage=fm;
         StyledDocument doc=m_preview.getStyledDocument();
         //if (isInline(fm.getFile().getPath())) {
             try {
                doc.remove(0,doc.getLength());
                addInline(doc,doc.getLength(),fm.getFile().getPath(),fm.getData());
             } catch (Exception blex) {
                 Logger.printException(blex);
             }
         //}
     }
     
     private void init() {
    	 m_mousie = new MouseHandler();
         setLayout(new BorderLayout());
         setBackground(Color.white);
         setOpaque(true);
         ExtActionMap map = new ExtActionMap(new ActionHandler());
         map.addAction("Preview.saveAttach", "Save attachment", "Svae the attachment for this post", null, "Ctrl S", 's');
         setActionMap(map);
         m_header=new CfHeaderPane();
         
         m_preview=new JTextPane();
         m_preview.addMouseListener(m_mousie);
         registerStyles(m_preview.getStyledDocument());
         m_preview.setEditable(false);
         add(m_header,BorderLayout.NORTH);
         add(m_preview,BorderLayout.CENTER);
     }
     
     private void setBody(Article a) {
         String mime= a.getHeader().getMimeType();
         String s=a.getBody().getContent();
         if ((mime!=null && mime.startsWith("text") && !mime.startsWith("text/plain")) && !s.endsWith("end")) {
             try {
            	StyledDocument doc= m_preview.getStyledDocument();
            	doc.remove(0,doc.getLength());
        	 	addInline(m_preview.getStyledDocument(), 0, mime, s.getBytes());
             } catch (BadLocationException blex) {
            	 
             }
         }  else if ((mime!=null && (mime.startsWith("application/octet-stream") || mime.startsWith("multipart"))) || s.startsWith("end")) {
            m_view.fireCoffeeEvent(m_view,CoffeeEvent.GET_ATTACHMENT,a);
         } else if ((s.indexOf("=ybegin"))!=-1 || (s.indexOf("\nbegin"))!=-1) {
          	m_view.fireCoffeeEvent(m_view,CoffeeEvent.GET_ATTACHMENT,a);
         } else {
             try {
                StyledDocument doc= m_preview.getStyledDocument();
                doc.remove(0,doc.getLength());
                StringTokenizer st = new StringTokenizer(s,"\n");
                int offset=0;
                while (st.hasMoreTokens()) {
                    String t=st.nextToken()+"\n";
                    int cnt=0;
                    while (t.charAt(cnt)=='>'|| (t.charAt(cnt)==' ' && t.charAt(cnt+1)=='>') ) cnt++;
                    if (cnt==0) {
                        doc.insertString(offset, t, doc.getStyle("default"));
                    } else {
                        doc.insertString(offset, t, doc.getStyle("gray"+(cnt>4?4:cnt)));
                    }
                    offset=doc.getLength();
                }
             } catch (BadLocationException blex) {
                Logger.printException(blex);
             }
             
         }
         
     }
     
     public Dimension getPreferredScrollableViewportSize() {
        return getSize();
    }
    public int getScrollableBlockIncrement(Rectangle visibleRect,
            int orientation, int direction) {
		return m_preview.getScrollableBlockIncrement(visibleRect,orientation,direction);
    }
    public boolean getScrollableTracksViewportHeight() {
       return false;
    }
    public boolean getScrollableTracksViewportWidth() {
        return true;
    }
    public int getScrollableUnitIncrement(Rectangle visibleRect,
            int orientation, int direction) {
        return m_preview.getScrollableUnitIncrement(visibleRect,orientation,direction);
    }
    
    private void registerStyles(StyledDocument doc) {
        
        Style def = StyleContext.getDefaultStyleContext().
                        getStyle(StyleContext.DEFAULT_STYLE);
        
        Style regular = doc.addStyle("regular", def);
        
        Style file= doc.addStyle("file",def);
        StyleConstants.setIcon(file,IconFactory.createIcon("/org/brains2b/coffee/resource/Text24.gif"));
        
        Style filename= doc.addStyle("filename",regular);
        StyleConstants.setFontSize(filename,9);
        
        Style button= doc.addStyle("button",def);
        JButton docbutton=new JButton(IconFactory.createIcon("/org/brains2b/coffee/resource/Text24.gif"));
       	docbutton.addMouseListener(m_mousie);
        docbutton.setActionCommand("Preview.default");
        docbutton.addActionListener(new ActionHandler());
        StyleConstants.setComponent(button,docbutton);
        
        doc.addStyle("document",def);
        
        for (int i=1;i<5;i++) {
            Style gray=doc.addStyle("gray"+i,def);
            StyleConstants.setForeground(gray, new Color(40+(int) (216d*0.20*i),20+(int) (75*0.20*i),2+(int)(10*0.20*i)));
        }
        
    }
    
    /*private boolean isInline(String filename) {
    	if (filename.toLowerCase().endsWith(".jpg")) {
    	    return true;
    	} else {
    	    return false;
    	}
    }*/
    
    private String getType(String filename) {
        int idx=filename.lastIndexOf('.');
        if (idx>=0) {
            return filename.substring(idx+1).toLowerCase();
        }
        return filename;
    }
    
    private void addInline(StyledDocument doc,int offset,String filename, byte[] b) throws BadLocationException {
        String type=getType(filename);
        if ("jpg".equals(type) || "jpeg".equals(type) || "png".equals(type)) {
            ImageIcon img= new ImageIcon(b);
            if (img.getIconWidth()>getSize().width) {
                int th=  (int) (img.getIconHeight() * ((double) getSize().width)/img.getIconWidth());
                img.setImage(img.getImage().getScaledInstance(getSize().width, th, Image.SCALE_DEFAULT));
            }
            
            StyleConstants.setIcon(doc.getStyle("file"),img);
            doc.insertString(offset," \n",doc.getStyle("file"));
            offset+=2;
            m_mousie.setPopupOn(true);
        } else if ("txt".equals(type) || "nfo".equals(type) || type.startsWith("text/") ) {
        	JEditorPane editor = new JEditorPane();
        	if (type.startsWith("text/")) {
        		editor.setContentType(type);
        	} else {
        		editor.setContentType("text/plain");
        	}
        	editor.addMouseListener(m_mousie);
        	m_mousie.setPopupOn(true);
        	editor.setEditable(false);
        	String content = new String(b)+"\n";
        	editor.setText(content);
        	StyleConstants.setComponent(doc.getStyle("document"),editor);
        	doc.insertString(offset," \n",doc.getStyle("document"));
        	offset+=2;
        	//offset+=content.length();
        } else {
            doc.insertString(offset," \n",doc.getStyle("button"));
            m_mousie.setPopupOn(true);
            offset+=2;
        }
        doc.insertString(offset,filename,doc.getStyle("filename"));
    }

    public Article getArticle() {
        return m_article;
    }
    
    
}
