package org.brains2b.coffee.gui.list;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.ListCellRenderer;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.brains2b.swingext.action.ExtActionMap;

public class ExtMutableList extends JPanel implements ActionListener {
	
	class ItemSelector implements ListSelectionListener {
	
	    public void valueChanged(ListSelectionEvent e) {
	        if (!e.getValueIsAdjusting() && e.getFirstIndex()>=0) {
	            getActionMap().get("ExtMutableList.Edit").setEnabled(true);
	            getActionMap().get("ExtMutableList.Remove").setEnabled(true);
	        }
	    }
	    
	}
	
	public final static int ACTION_ADD=1;
	public final static int ACTION_REMOVE=2;
	public final static int ACTION_EDIT=4;
	
	private JList m_list;
	private int m_availableActions;
	
	public ExtMutableList() {
		this(ACTION_ADD+ACTION_REMOVE+ACTION_EDIT);
	}
	
	public ExtMutableList(int availableActions) {
		super();
		m_availableActions =availableActions;
		init();
	}
	
	public void setFixedCellWidth(int width) {
		m_list.setFixedCellWidth(width);
	}
	
	public void setFixedCellHeight(int height) {
		m_list.setFixedCellHeight(height);
	}
	
	public void setVisibleRowCount(int visibleRowCount) {
		m_list.setVisibleRowCount(visibleRowCount);
	}
	
	public List getData() {
		//TODO should not use member directly
		return ((MutableListModel) m_list.getModel()).m_data;
	}
	
	public void setData(List l) {
		((MutableListModel) m_list.getModel()).setData(l);
	}
	
	public void clearData() {
		//TODO should not use member directly
		if (((MutableListModel) m_list.getModel()).m_data!=null) {
			((MutableListModel) m_list.getModel()).m_data.clear();
		}
	}
	
	public void setData(Object[] array) {
		setData(Arrays.asList(array));
	}
	
	public void addItem(Object o) {
		((MutableListModel) m_list.getModel()).addData(o);
	}

	public void replaceItem(Object o, int index) {
		((MutableListModel) m_list.getModel()).m_data.set(index, o);
	}
	
	public void removeItem(int index) {
		((MutableListModel) m_list.getModel()).removeData(index);
	}
	
	public void setCellRenderer(ListCellRenderer cellRenderer) {
		m_list.setCellRenderer(cellRenderer);
	}
	
	public Object getSelectedValue() {
		return m_list.getSelectedValue();
	}
	
	public int getSelectedIndex() {
		return m_list.getSelectedIndex();
	}
	
	
	
	public void actionPerformed(ActionEvent e) {
		ActionListener[] listeners = (ActionListener[]) getListeners(ActionListener.class);
		for (int i=0;i<listeners.length;i++) {
			listeners[i].actionPerformed(e);
		}
	}
	
	public void addListSelectionListener(ListSelectionListener listener) {
		m_list.addListSelectionListener(listener);
	}
	
	public void removeListSelectionListener(ListSelectionListener listener) {
		m_list.removeListSelectionListener(listener);
	}
	
	public void addActionListener(ActionListener listener) {
		listenerList.add(ActionListener.class, listener);
	}
	
	public void removeActionListener(ActionListener listener) {
		listenerList.remove(ActionListener.class, listener);
	}
	
	private void init() {
		ResourceBundle rs = ResourceBundle.getBundle("org.brains2b.coffee.resource.coffee");
		setLayout(new BorderLayout()); 
        m_list= new JList(new MutableListModel());
        m_list.addListSelectionListener(new ItemSelector());
        add(new JScrollPane(m_list),BorderLayout.CENTER);
        
        ExtActionMap map = new ExtActionMap(this);
        map.addAction("ExtMutableList.Add",rs.getString("ExtMutableList.Add"), rs.getString("ExtMutableList.Add.Tooltip"),
                "/org/brains2b/coffee/resource/Plus16.gif", "+", '+');
        map.addAction("ExtMutableList.Edit",rs.getString("ExtMutableList.Edit"), rs.getString("ExtMutableList.Edit.Tooltip"),
                "/org/brains2b/coffee/resource/Edit16.gif",  "+", '+');
        map.addAction("ExtMutableList.Remove",rs.getString("ExtMutableList.Remove"), rs.getString("ExtMutableList.Remove.Tooltip"),
                "/org/brains2b/coffee/resource/Minus16.gif","+", '+');
        setActionMap(map);
                
        JToolBar actions= new JToolBar(SwingConstants.VERTICAL);
        actions.setFloatable(false);
        if ((m_availableActions & ACTION_ADD)==ACTION_ADD)
        actions.add(getActionMap().get("ExtMutableList.Add"));
        if ((m_availableActions & ACTION_EDIT)==ACTION_EDIT)
        actions.add(getActionMap().get("ExtMutableList.Edit"));
        if ((m_availableActions & ACTION_REMOVE)==ACTION_REMOVE)
        actions.add(getActionMap().get("ExtMutableList.Remove"));
        
        //Wait for selection
        map.get("ExtMutableList.Edit").setEnabled(false);
        map.get("ExtMutableList.Remove").setEnabled(false);
        
        add(actions,BorderLayout.EAST);
	}
	
}
