/* 	ArticleHeader.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 17-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.message;

import java.util.Date;

import org.brains2b.coffee.util.CfDateHelper;
import org.brains2b.log.Logger;
import org.brains2b.saxo.SAXWriter;
import org.brains2b.util.XmlHelper;
import org.xml.sax.SAXException;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class ArticleHeader implements NNTPMessage,Cloneable {
    
    public final static String[] TYPES = new String[] 
        {"Subject","From","Date","Content-Type","Lines","Bytes","Xref","Newsgroups","References"};
    
    private Object[] m_values;
    //private HashMap m_header;
    /**
     * 
     */
    public ArticleHeader() {
        super();
        m_values=new Object[TYPES.length];
    }

    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#addLine(java.lang.String)
     */
    public void addLine(String s) {
        int idx=s.indexOf(':');
        if (idx>0) {
            String key=s.substring(0,idx);
            String value=s.substring(idx+1).trim();
            addHeaderValue(key,value);
        }

    }

    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#addHeader(int, java.lang.String)
     */
    public void addHeader(int id, String s) {
        
    }

    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#isHeaderSet()
     */
    public boolean isHeaderSet() {
        return false;
    }

    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#endMessage()
     */
    public void endMessage() {
        
    }

    public int getType() {
        return 220;
    }
    
    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#writeMessage(org.brains2b.saxo.SAXWriter)
     */
    public void writeMessage(SAXWriter sw) throws SAXException {
        writeHeader(sw);
    }
    
    public void writeHeader(SAXWriter sw) throws SAXException {
       
        sw.startElement("header",null);
        for (int i=0;i<TYPES.length;i++) {
            if (m_values[i]!=null) {
                sw.startElement(TYPES[i],null);
                if ("Date".equals(TYPES[i])) {
                    sw.setContent(XmlHelper.encode( CfDateHelper.format(getDate())));
                } else {
                    sw.setContent(XmlHelper.encode( (String) m_values[i]));
                }
                sw.endElement(TYPES[i]);
            }
        }
        sw.endElement("header");
    }
    
    public void addHeaderValue(String field, String value) {
        int idx=getIndex(field);
        if (idx>-1) {
            if (idx==2) {
                m_values[idx]=CfDateHelper.parse(value);
            } else {
                m_values[idx]=value;
            }
        } else {
            Logger.println(field + ": " + value);
        }
        
    }

    public String getHeaderValue(String field) {
        return (String) getHeaderObject(field);
    }
    
    public Object getHeaderObject(String field) {
        int idx=getIndex(field);
        if (idx>-1) {
            return m_values[idx];
        }
        return null;
    }
    
    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#hasContent()
     */
    public boolean hasContent() {
        return true;
    }
    
    public String getSubject() {
        return (String) m_values[0];
    }
    
    public void setSubject(String subject) {
        m_values[0]=subject;
    }
    
    public String getAuthor() {
        return (String) m_values[1];
    }
    
    public Date getDate() {
        return (Date) m_values[2];
    }
    
    public int getLines() {
        String s=(String) m_values[4];
        if (s!=null) {
            try {
                return Integer.parseInt(s);
            } catch (NumberFormatException nfex) {}
        }
        return 0;
    }
    
    public String getMimeType() {
        return (String) m_values[3];
    }
    
    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#getSize()
     */
    public int getSize() {
        String s=(String) m_values[5];
        if (s!=null) {
            try {
                return Integer.parseInt(s);
            } catch (NumberFormatException nfex) {}
        }
        return 0;
    }
    
    private final int getIndex(String field) {
        int idx=-1;
        for (int i=0;i<TYPES.length;i++) {
            if (TYPES[i].equals(field))  {
                idx=i;
                break;
            }
        }
        return idx;
    }
    public String getId() {
        return "header";
    }
    
    public Object clone()  {
    	ArticleHeader h= new ArticleHeader();
    	System.arraycopy(m_values, 0, h.m_values, 0, TYPES.length);
    	return h;
    }
}
