/* 	MultiPartFilter.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 27-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.message;

import java.util.ArrayList;

import org.brains2b.coffee.util.SubjectHelper;
import org.brains2b.coffee.util.SubjectInfo;
import org.brains2b.log.Logger;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class MultiPartGroup extends NewsGroup {
	
    public MultiPartGroup() {
        super();
        m_articleList=new ArrayList();
    }
    
     public MultiPartGroup(NewsGroup group) {
        super();
        setAllowPost(group.isAllowPost());
        setEstimate(group.getEstimate());
        setFirst(group.getFirst());
        setLast(group.getLast());
        setLastUpdated(group.getLastUpdated());
        setName(group.getName());
        m_articleList=group.getArticleList()==null?new ArrayList():group.getArticleList();
    }
     
     public void addLine(String s) {
    	if (getType()==224) {
    		Article a = new Article();
    		a.addLine(s);
    		filterArticle(a);
    	} else {
    		super.addLine(s);
    	}
    }
     
    public void filterArticle(Article a) {
        if (!a.isNew()) {
            addArticle(a);
        } else {
           String subject=a.getHeader().getSubject();
           if (subject.toUpperCase().indexOf("YENC")!=-1) {
               a.getHeader().addHeaderValue("Content-Type","x-application/yEnc");
           }
           
           final SubjectInfo fsi=SubjectHelper.getTruncated(subject);
           if (fsi!=null) {
               boolean found=false;
               for (int i=0;i<m_articleList.size();i++) {
                   Article fa=(Article) m_articleList.get(i);
                   String fns=fa.getSubject();
                   if (fns!=null && fns.equals(fsi.getSubject())) {
                        if (fa.getPart()>fsi.getPart()) {
                            fa.addPart(fa.getPart()-2,fa.getId());
                            fa.setPart(fsi.getPart());
                            fa.setHeader(a.getHeader());
                            fa.setSubject(fsi.getSubject());
                            fa.getHeader().addHeaderValue("Bytes", String.valueOf(fa.getSize()+a.getSize()));
                            fa.setId(a.getId());
                            fa.setNumber(a.getNumber());
                        } else {
                            try {
                                fa.addPart(fsi.getPart()-2,a.getId());
                                fa.getHeader().addHeaderValue("Bytes", String.valueOf(fa.getSize()+a.getSize()));
                            } catch (ArrayIndexOutOfBoundsException aioex) {
                                Logger.printException(aioex);
                            }
                        }
                        found=true;
                        break;
                   }
               }
               if (!found) {
                   a.setSubject(fsi.getSubject());
                   a.setTotalParts(fsi.getTotal());
                   a.setPart(fsi.getPart());
                   addArticle(a);
               }
           }  else {
               addArticle(a);
           }
        }
        
    }
    
}
