/* 	FilterList.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 20-aug-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.message.filter;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.brains2b.coffee.message.GroupList;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.saxo.SAXWriter;
import org.brains2b.util.XmlHelper;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

public class FilterList implements NNTPMessage {
    public final static int FILTER_LIST=1301;
	private final static FilterList m_instance=new FilterList(); 
    private static boolean m_loaded;
    
    private HashMap m_filters;

    protected FilterList() {
        m_filters=new HashMap();   
    }
        
    public static FilterList getInstance() {
        return m_instance;
    }
    
    public List getFilterNames() {
    	return new ArrayList(m_filters.keySet());
    }
    
    public List getAllFilterFor(String key) {
        ArrayList l = new ArrayList();
        List lAll = (List) m_filters.get("*");
        List lKey = (List) m_filters.get(key);
    	if (lAll!=null) l.addAll(lAll);
    	if (lKey!=null) l.addAll(lKey);
    	return l;
    }
    
    public List getFilterFor(String key) {
        List lKey = (List) m_filters.get(key);
    	return lKey;
    }
    
    public void setFilterFor(String key, List filters) {
        m_filters.put(key, filters);
    }
    
    public void addFilterFor(String key, MessageFilter filter) {
        List  list = (List) m_filters.get(key);
        if (list == null) {
        	list = new ArrayList();
        	m_filters.put(key, list);
        }
    	list.add(filter);
    }
    
    public int getSize() {
        return m_filters.size();
    }

    public int getType() {
        return FILTER_LIST;
    }

    public boolean hasContent() {
        return m_loaded;
    }

    public boolean isHeaderSet() {
        return false;
    }

    public void writeMessage(SAXWriter sw) throws SAXException {
        sw.startElement("filters",null);
        Iterator it=m_filters.keySet().iterator();
        while (it.hasNext()) {
            String key= (String) it.next();
            List list= (List) m_filters.get(key);
            AttributesImpl atts=new AttributesImpl();
            if (key.equals(GroupList.class.getName())) {
               atts.addAttribute(null,null,"on","CDATA","group");    
            } else {
                atts.addAttribute(null,null,"on","CDATA","message");
                atts.addAttribute(null,null,"id","CDATA",key);
            }
            sw.startElement("filter",atts);
            for (Iterator itf = list.iterator();itf.hasNext();) {
                MessageFilter mf =(MessageFilter) itf.next();
                AttributesImpl qAtts=new AttributesImpl();
                qAtts.addAttribute(null,null,"field","CDATA",mf.getKey());
                qAtts.addAttribute(null,null,"operation","CDATA",MessageFilter.OPERATIONS[mf.getOperator()]);
                qAtts.addAttribute(null,null,"not","CDATA",String.valueOf(mf.isNot()));
                qAtts.addAttribute(null,null,"kill","CDATA",String.valueOf(mf.isKill()));
                sw.startElement("query",qAtts);
                sw.setContent(XmlHelper.encode(mf.getValue().toString()));
                sw.endElement("query");
            }
            sw.endElement("filter");
        }
        sw.endElement("filters");
    }
    
    public String getId() {
        return "filters";
    }
    
    //Local message so not implemented
    public void addHeader(int id, String s) {
    }

    public void addLine(String s) {
    }

    public void endMessage() {
    }

}
