/* 	FilterNewsGroup.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 7-aug-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.message.filter;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.NewsGroup;

public class FilterNewsGroup extends NewsGroup implements FilterTarget {
    private NewsGroup m_group;
    private List m_filters;
    private int m_size;
    
    public FilterNewsGroup(NewsGroup group, List filters) {
        m_group=group;
        m_filters=filters;
    }
    
    public List getArticleList() {
    	
        List articles=m_group.getArticleList();
        if (articles==null) return null;
        List out=new ArrayList(articles.size()/2);
        for (Iterator it = articles.iterator(); it.hasNext();) {
        	Article a = (Article) it.next();
        	boolean add=true;
        	boolean remove=false;
        	for (Iterator itf = m_filters.iterator();itf.hasNext();) {
        		MessageFilter mf = (MessageFilter) itf.next();
        		if (mf.getKey().equals("new")) {
        			add = add && a.isNew();	
        		} else { 
        			Object value =a.getHeader().getHeaderObject(mf.getKey());
        			add = add && mf.isValid(value);
        		}
        		remove =remove || (!add && mf.isKill());
        	}
        	if (add) out.add(a);
        	if (remove) it.remove();
        }
        
        m_size=out.size();
        return out;
    }

    public int getFilteredSize() {
        return m_size;
    }
    
    public int getSize() {
        return m_group.getSize();
    }
    
    public String getName() {
        return m_group.getName();
    }

	public int getEstimate() {
		return m_group.getEstimate();
	}

	public int getFirst() {
		return m_group.getFirst();
	}

	public String getId() {
		return m_group.getId();
	}

	public int getLast() {
		return m_group.getLast();
	}

	public Date getLastUpdated() {
		return m_group.getLastUpdated();
	}

	public int getType() {
		return m_group.getType();
	}

	public boolean hasContent() {
		return m_group.hasContent();
	}

	public boolean isAllowPost() {
		return m_group.isAllowPost();
	}

	public void removeArticle(Article a) {
		m_group.removeArticle(a);
	}

	public void setAllowPost(boolean allowPost) {
		m_group.setAllowPost(allowPost);
	}

	public void setArticleList(Vector articleList) {
		m_group.setArticleList(articleList);
	}

	public void setEstimate(int estimate) {
		m_group.setEstimate(estimate);
	}

	public void setFirst(int first) {
		m_group.setFirst(first);
	}

	public void setLast(int last) {
		m_group.setLast(last);
	}

	public void setLastUpdated(Date lastUpdated) {
		m_group.setLastUpdated(lastUpdated);
	}

	public void setName(String name) {
		m_group.setName(name);
	}
}
