/* 	MimeHandler.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 13-jan-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.plugin;

import java.util.ArrayList;
import java.util.Iterator;

import org.brains2b.coffee.CoffeeSpill;
import org.brains2b.coffee.TicketHandler;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.gui.CfFrame;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.local.FileMessage;
import org.brains2b.coffee.monitor.ProgressMonitor;

public class MimeHandler implements TicketHandler {

    
    private final static int[] EVENTS = new int[] {
        CoffeeEvent.USE
    };
    
    private CfFrame m_manager;
    private ArrayList m_plugins;    
    public MimeHandler(CfFrame manager) {
        super();
        m_manager = manager;
        m_plugins = new ArrayList();
    }
    
    
    public void addProgressMonitor(ProgressMonitor m) {
        // TODO Auto-generated method stub

    }

    public void close() throws Exception {
        // TODO Auto-generated method stub

    }

    public int[] getHandledEvents() {
        return EVENTS;
    }

    public NNTPMessage handleEvent(CoffeeEvent event) throws CoffeeSpill {
        FileMessage fm = (FileMessage) event.getArgument();
        MimePlugin mp = findPlugin(fm.getMimeType(),event.getID());
        if (mp!=null && mp instanceof CoffeeExchangeView) {
            m_manager.showPlugin(mp.getShortTitle());
            ((CoffeeExchangeView)mp).setCurrentMessage(fm);
        }
        return null;
    }

    public boolean isAvailable() {
        // TODO Auto-generated method stub
        return true;
    }

    public void open() throws Exception {
        // TODO Auto-generated method stub

    }

    public void removeProgressMonitor(ProgressMonitor m) {
        // TODO Auto-generated method stub

    }

    public void registerPlugin(CoffeeExchangeView view) {
        m_manager.registerPlugin(((MimePlugin) view).getShortTitle(), view);
        m_plugins.add(view);
    }
    
    public void registerPlugin(CoffeeExchangeControl control) {
        m_plugins.add(control);
    }
    
    private MimePlugin findPlugin(String mimeType, int action) {
        for (Iterator it = m_plugins.iterator();it.hasNext();) {
            MimePlugin plugin = (MimePlugin) it.next();
            if (plugin.isActionSupported(mimeType, action)) {
                return plugin;
            }
        }
        return null;
    }
}
