/* 	NzbGuiPanel.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 29-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.plugin.nzb;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import org.brains2b.coffee.Ticket;
import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.local.CoffeeMenu;
import org.brains2b.coffee.message.local.FileMessage;
import org.brains2b.coffee.message.local.ServerDescriptor;
import org.brains2b.coffee.plugin.CoffeeExchangeView;
import org.brains2b.coffee.plugin.MimePlugin;
import org.brains2b.file.PostFixFilter;
import org.brains2b.swingext.action.ExtActionMap;
import org.brains2b.swingext.gui.IconFactory;
import org.brains2b.task.TaskEvent;
import org.brains2b.task.TaskListener;


/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class NzbGuiPanel extends JPanel implements CoffeeExchangeView, MimePlugin, ActionListener {
	
	class SelectionHandler implements TreeSelectionListener {

		public void valueChanged(TreeSelectionEvent e) {
			NzbDisplay display = (NzbDisplay) e.getPath().getLastPathComponent();
			if (!(display.getContent() instanceof NzbArticle)) {
				getActionMap().get("nzb.remove").setEnabled(true);
				if (display.getContent().isPaused()) {
					getActionMap().get("nzb.resume").setEnabled(true);
					getActionMap().get("nzb.pause").setEnabled(false);
				} else {
					getActionMap().get("nzb.resume").setEnabled(false);
					getActionMap().get("nzb.pause").setEnabled(true);
				}
			}
		}
		
	}
    private class TaskHandler implements TaskListener {
        /* (non-Javadoc)
         * @see org.brains2b.task.TaskListener#taskProgressed(org.brains2b.task.TaskEvent)
         */
        public void taskProgressed(TaskEvent e) {
            if (e.getSource() instanceof Ticket) {
	            Ticket t=(Ticket) e.getSource();
                NzbDocument doc= ((NzbProcessor) t.getEvent().getSource()).m_doc;
		        if (e.isFinished()) {
                    Article a=(Article) t.getEvent().getArgument();
                    if (t.getMessage() instanceof Article) {
                        a=(Article) t.getMessage();
                    }
		            NzbArticle na=doc.findArticle(a);
		            NzbFile nf=doc.findFile(a);
		         	if (na!=null) {
		         	    NzbDisplay d=findDisplay(m_treeRoot,na);
		       
			         	if (d!=null) {
				         	d.setValue(na.getSize());
			         	}
			         	d=findDisplay(m_treeRoot,nf);
			         	if (d!=null) { 
			         		d.setValue(d.getValue()+1);
                            
                            if (d.getValue()==d.getMax()) {
                                d=findDisplay(m_treeRoot,doc);
                                if (d!=null) {
                                    d.setValue(d.getValue()+1);
                                }
                            }
			         	}
                        m_tree.repaint();
		         	}
		         	
		        }
            }

        }
        
   	    	
    }
    
    private class MouseHandler extends MouseAdapter {
        
        public void mousePressed(MouseEvent e) {
            popupMouse(e);
            super.mousePressed(e);
        }
        
        public void mouseReleased(MouseEvent e) {
            popupMouse(e);
            super.mouseReleased(e);
        }
        
        private void popupMouse(MouseEvent e) {
            if (!e.isPopupTrigger()) return;
            JPopupMenu menu=new JPopupMenu();
            JTree tree= (JTree) e.getSource();
            TreePath path= tree.getPathForLocation(e.getX(), e.getY());
            tree.setSelectionPath(path);
            menu.add(getActionMap().get("nzb.remove")); 
            menu.add(getActionMap().get("nzb.pause"));
            menu.add(getActionMap().get("nzb.resume"));
            if (menu.getComponentCount()>0) {
                menu.show(tree, e.getX(), e.getY());
            }
        }
    }
    
    private class NzbTreeComponent implements TreeCellRenderer {
        JPanel panel;
        JLabel label;
        JProgressBar progressBar;
		public NzbTreeComponent() {
		    panel=new JPanel();
            panel.setOpaque(false);
		    label=new JLabel();
            progressBar=new JProgressBar();
            progressBar.setOpaque(false);
            progressBar.setPreferredSize(new Dimension(60,16));
            progressBar.setMaximumSize(new Dimension(60,16));
            progressBar.setString("Que");
            progressBar.setStringPainted(true);
            panel.add(label);
            panel.add(progressBar);
        }
        
       	public Component getTreeCellRendererComponent(JTree tree, Object value,
        	        boolean selected, boolean expanded, boolean leaf, int row,
            	    boolean hasFocus) {
       	    NzbDisplay node= (NzbDisplay) value;
            NzbContent content = node.getContent();
        	if (node!=null) { 
        		if (row>0) {
	        		if (content instanceof NzbDocument) {
	        			label.setIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Folderbin.gif"));
	        		} else if (content instanceof NzbFile) {
	        			label.setIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Binary16.gif"));
	        		} else {
	        			label.setIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/BinaryBroken16.gif"));
	        		}
        		}
       	    	label.setText(node.getSubject());
                if (selected) {
                    label.setBackground(UIManager.getColor("ProgressBar.backgroundHighlight"));
                    label.setOpaque(true);
                } else {
                    label.setOpaque(false);
                }
                if (content==null) {
                    progressBar.setVisible(false);
                    return panel;
                } else {
                    progressBar.setVisible(true);
                }
                progressBar.setBackground(UIManager.getColor("ProgressBar.background"));
                progressBar.setForeground(UIManager.getColor("ProgressBar.foreground"));
                progressBar.setString("Que");
                progressBar.setMaximum(node.getMax());
                progressBar.setValue(node.getValue());
                if (content.isPaused()) {
                	progressBar.setString("Paused");
                } else if (content.isComplete()) {
                    progressBar.setString(content.isFailed()?"Failed":"Done!");
                } else if (node.getValue()>1 && node.getMax()!=node.getValue()) {
                    progressBar.setString("Running");
                } 
        		return panel;
        	} else {
        	    return new JLabel();
        	}
       	    
        }
	}
    
    private class NzbDisplay implements MutableTreeNode {
	    private ArrayList m_children;
        private NzbContent m_content;
        private NzbDisplay m_parent;
	    private int m_value;
	    
        public NzbDisplay() {
           m_children = new ArrayList(); 
        }
        
	    public NzbDisplay(NzbContent content) {
	        m_content=content;
            if (!(m_content instanceof NzbArticle)) {
                m_children = new ArrayList();
            }
	    }
	    
	    public String getSubject() {
            if (m_content==null) return "";
	        return m_content.getSubject();
	    }
	    
        public int getValue() {
            if (m_content!=null && m_content.isComplete()) return m_content.getSize();
            return m_value;
        }
        public void setValue(int value) {
            m_value = value;
        }
        public int getType() {
            return m_content.getType();
        }
       
        public int getMax() {
            return m_content==null?0:m_content.getSize();
        }
        
        public NzbContent getContent() {
            return m_content;
        }

        public Enumeration children() {
            return Collections.enumeration(m_children);
        }

        public boolean getAllowsChildren() {
            return !(m_content instanceof NzbArticle);
        }

        public TreeNode getChildAt(int childIndex) {
            return (TreeNode) m_children.get(childIndex);
        }

        public int getChildCount() {
            return m_children.size();
        }

        public int getIndex(TreeNode node) {
            return m_children.indexOf(node);
        }

        public TreeNode getParent() {
            return m_parent;
        }

        public boolean isLeaf() {
            return (m_content instanceof NzbArticle) || m_children.isEmpty();
        }

        public void insert(MutableTreeNode child, int index) {
            m_children.add(index, child);
            child.setParent(this);
        }
        
        public void add(MutableTreeNode child) {
            m_children.add(child);
            child.setParent(this);
        }

        public void remove(int index) {
            m_children.remove(index);
        }

        public void remove(MutableTreeNode node) {
            m_children.remove(node);
        }

        public void removeFromParent() {
            m_parent.remove(this);
        }

        public void setParent(MutableTreeNode newParent) {
            m_parent=(NzbDisplay) newParent;
        }

        public void setUserObject(Object object) {
            m_content = (NzbContent) object;
        }
	}
    private NzbDisplay m_treeRoot;
	private JTree m_tree;
	private ServerDescriptor m_server;
    /**
     * 
     */
    public NzbGuiPanel() {
		super();
        init();
    }
    
    private void init() {
        setLayout(new BorderLayout());
        JToolBar selPnl=new JToolBar();
        
        ExtActionMap map =new ExtActionMap(this);
        map.addAction("nzb.openFile","open nzb-document", "open an nzb file", "/org/brains2b/coffee/resource/Open16.gif", "Ctrl Shift O", 'o');
        map.addAction("nzb.remove","remove nzb-part", "remove this nzb part", "/org/brains2b/coffee/resource/Remove16.gif","Ctrl Shift R", 'r');
        map.addAction("nzb.pause","pause nzb-part", "pause this nzb part", "/org/brains2b/coffee/resource/Pause16.gif","Ctrl P", 'p');
        map.addAction("nzb.resume","resume nzb-part", "resume this nzb part", "/org/brains2b/coffee/resource/Play16.gif","Ctrl Shift P", 'p');
        setActionMap(map);
        map.get("nzb.remove").setEnabled(false);
        JButton btn= new JButton(map.get("nzb.openFile"));
        btn.setText(null);
        selPnl.add(btn);
        selPnl.addSeparator();
        btn = new JButton(map.get("nzb.remove"));
        btn.setText(null);
        selPnl.add(btn);
        btn = new JButton(map.get("nzb.pause"));
        btn.setText(null);
        selPnl.add(btn);
        btn = new JButton(map.get("nzb.resume"));
        btn.setText(null);
        selPnl.add(btn);
        
        add(selPnl,BorderLayout.NORTH);
        JScrollPane scroll=new JScrollPane();
        JTree tree=new JTree();
        scroll.setViewportView(tree);
        //tree.setRootVisible(false);
        m_treeRoot=new NzbDisplay();
        tree.setModel(new DefaultTreeModel(m_treeRoot));
        tree.setCellRenderer(new NzbTreeComponent());
        tree.getSelectionModel().addTreeSelectionListener(new SelectionHandler());
        tree.addMouseListener(new MouseHandler());
        add(scroll,BorderLayout.CENTER);
        m_tree=tree;
    }
    
    public void activate() {
    	if (m_server==null) {
        	m_server = TicketBroker.getBroker().getCurrentServer();
        	if (m_server==null) {
        		fireCoffeeEvent(this, CoffeeEvent.GET_LIST, "server");
        	}
    	}

    }
    public void deactivate() {
        // TODO Auto-generated method stub

    }
    public void fireCoffeeEvent(CoffeeExchange source, int action, Object argument) {
        TicketBroker.getBroker().broker(new CoffeeEvent(source,action,argument));

    }
    public JMenu[] getMenus() {
        // TODO Auto-generated method stub
        return null;
    }
    
    public void setCurrentMessage(NNTPMessage message) {
       	if (message instanceof FileMessage) {
       	    FileMessage fm=(FileMessage) message;
       	    if ("text/xml.nzb".equals(fm.getMimeType())) {
       	        if (!fm.hasContent()) {
       	        	fireCoffeeEvent(this,CoffeeEvent.XML_READ,fm);
       	        } else {
       	            String name=fm.getFile().getAbsolutePath();
       	            int idx=name.lastIndexOf(System.getProperty("file.separator"));
       	            if (idx>0) {
       	                name=name.substring(idx+1);
       	            } 
       	            NzbDocument doc= (NzbDocument) fm.getContent();
       	            doc.setSubject(name);
       	            ServerDescriptor sd = TicketBroker.getBroker().getCurrentServer();
       	            if (sd==null || !sd.isBinary()) {
       	            	sd = m_server;
       	            }
       	            NzbProcessor processor=new NzbProcessor(doc,new File(sd.getDownload()));
       	            createTree(doc);
                    processor.addTaskListener(new TaskHandler());
       	            fireCoffeeEvent(this,CoffeeEvent.PROCESS,processor);
       	            
       	        }
       	       
       	    } 
       	} else if ( message instanceof CoffeeMenu && message.getId().equals("server")) {
   			NNTPMessage[] servers = ((CoffeeMenu) message).getArray();
   			for (int i=0;i<servers.length;i++) {
   				if (((ServerDescriptor) servers[i]).isBinary()) {
   					m_server = (ServerDescriptor) servers[i];
   					fireCoffeeEvent(this, CoffeeEvent.CHANGE_SERVER, m_server);
   					break;
   				}
   			}
       	}

    }
    public void createTree(NzbDocument document) {
        NzbDisplay doc = new NzbDisplay(document);
        NzbFile[] files=document.getFiles();
        for (int i=0;i<files.length;i++) {
            NzbDisplay file=new NzbDisplay(files[i]);
            doc.add(file);
            NzbArticle[] articles=files[i].getArticles();
            for (int j=0;j<articles.length;j++) {
                file.add(new NzbDisplay(articles[j]));
            }
        }
        m_treeRoot.add(doc);
        ((DefaultTreeModel) m_tree.getModel()).reload();
    }
    
    public NzbDisplay findDisplay(NzbDisplay node,NzbContent content) {
        NzbDisplay display=null;
        int size=node.getChildCount();
        for (int i=0;i<size;i++) {
            NzbDisplay child= (NzbDisplay) node.getChildAt(i);
            
            if (content.equals(child.getContent())) {
                display= child;
                break;
            } else if (!child.isLeaf()) {
               	display = findDisplay(child,content);
               	if (display!=null) {
               	    break;
               	}
            }
        }
        return display;
    }
    
    public String getShortTitle() {
        return "Newsbin";
    }
    
    public boolean isActionSupported(String mime, int action) {
        if (mime.equals("text/xml.nzb") && action == CoffeeEvent.USE) {
            return true;
        }
        return false;
    }

    public void actionPerformed(ActionEvent e) {
    	String actionCommand = e.getActionCommand();
          if ("nzb.openFile".equals(actionCommand)) {
            JFileChooser jfc=new JFileChooser();
            jfc.setFileFilter(new PostFixFilter(".nzb","Newzbin files"));
            jfc.showOpenDialog(NzbGuiPanel.this);
            if (jfc.getSelectedFile()!=null) {
                File f= jfc.getSelectedFile();
                if (f.exists()) {
                    FileMessage fm=new FileMessage();
                    fm.setFile(f);
                    fm.setMimeType("text/xml.nzb");
                    fireCoffeeEvent(NzbGuiPanel.this,CoffeeEvent.XML_READ,fm);
                }
            }
        } else if ("nzb.remove".equals(actionCommand)) {
            NzbDisplay nd=(NzbDisplay) m_tree.getSelectionPath().getLastPathComponent();
            nd.removeFromParent();
            handleTickets(nd.getContent(),CoffeeEvent.QUEUE_REMOVE);
            ((DefaultTreeModel) m_tree.getModel()).nodeStructureChanged((NzbDisplay) m_tree.getSelectionPath().getParentPath().getLastPathComponent());
        } else if ("nzb.pause".equals(actionCommand)) {
        	NzbDisplay nd=(NzbDisplay) m_tree.getSelectionPath().getLastPathComponent();
            handleTickets(nd.getContent(),CoffeeEvent.QUEUE_PAUSE);
            nd.getContent().setPaused(true);
            ((DefaultTreeModel) m_tree.getModel()).nodeChanged(nd);
        } else if ("nzb.resume".equals(actionCommand)) {
        	NzbDisplay nd=(NzbDisplay) m_tree.getSelectionPath().getLastPathComponent();
            handleTickets(nd.getContent(),CoffeeEvent.QUEUE_RESUME);
            nd.getContent().setPaused(false);
            ((DefaultTreeModel) m_tree.getModel()).nodeChanged(nd);
        }
        
    }
    
    private void handleTickets(NzbContent content, int action) {
    	 List parts = getParts(content);
    	 List tickets = TicketBroker.getBroker().getQueList();
    	 List actOn = new ArrayList();
    	 for (Iterator it =tickets.iterator();it.hasNext();) {
    		 Ticket t = (Ticket) it.next();
    		 Object o =t.getEvent().getArgument();
    		 if ( o instanceof NNTPMessage && parts.contains(((NNTPMessage)o).getId())) {
   				actOn.add(t);
    		 }
    	 }
    	 if (!actOn.isEmpty()) {
    		 fireCoffeeEvent(this, action, actOn.toArray(new Ticket[actOn.size()]));
    	 }
    }
    
    private List getParts(NzbContent content) {
    	List l = new ArrayList();
    	if (content instanceof NzbDocument) {
    		NzbFile[] files = ((NzbDocument) content).getFiles();
    		for (int i=0;i<files.length;i++) {
    			l.addAll(getParts(files[i]));
    		}
    	} else if (content instanceof NzbFile) {
    		NzbArticle[] articles = ((NzbFile)content).getArticles();
    		for (int i=0;i<articles.length;i++) {
    			l.add(articles[i].getId());
    		}
    	} else if (content instanceof NzbArticle) {
    		l.add(((NzbArticle) content).getId());
    	}
    	return l;
    }
    
}
