/* 	Yenc.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 17-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * TODO Use TreeSet directly
 * TODO Implement CRC check
 * TODO What happens when there is other text in message?
 */
/* @#History
 * [When]		[Who]					[What]
 * May 8, 2007	dennis@brains2b.nl      Read correct content type ISO-8559-1 from outset (don't convert here)        		
*/
package org.brains2b.coffee.util;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.Vector;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class Yenc {
	
    private RandomAccessFile m_temp;
    
    private class Descriptor implements Comparable {
        int part;
        int line;
        int size;
        int total;
        int begin;
        int end;
        byte[] data;
        String name;
        String crc;
        boolean stored=false;
        boolean finished=false;;
        
        public int compareTo(Object o) {
            int result=-1;
            if (o instanceof Descriptor) {
            	result=part-((Descriptor)o).part;
            }
           return result;
        }
    }
    //private String m_fileName;
    public Vector m_parts;
    /**
     * 
     */
    public Yenc() {
        super();
        m_parts=new Vector();
    }
    
    public void decodePiece(String s) throws Exception {
       //Charset cs = Charset.forName("ISO-8859-1");
       //CharBuffer cb = cs.decode(ByteBuffer.wrap(s.getBytes()));
       Descriptor d=_decode(s.toCharArray());
       if (d.size>2*1024*1024) {
           store(d);
       }
       m_parts.add(d);
    }
    
    public String encode(File f, int part, int size) {
    	try {
    		BufferedInputStream bis = new BufferedInputStream(new FileInputStream(f));
    		boolean useParts=false;
    		Descriptor d = new Descriptor();
    		d.name = f.getName();
	    	d.total = (int) f.length();
	    	d.size = size;
	    	if (d.total>d.size || part>1) {
	    		useParts=true;
	    	} 
	    	d.part = part;
	    	d.line = 128;
	    	char[] cb = new char[(int) ((double)d.size*1.32d)];
	    	int b = -1;
	    	int cnt =0;
	    	int biteCnt=0;
	    	int line = 0;
	    	cnt = createHeader(d,cb,useParts);
	    	cnt = useParts?createPartHeader(d,cb,cnt):cnt;
	    	if (part>1) bis.skip(d.size*(part-1)-1);
	    	while ((b=bis.read())!=-1 && ++biteCnt<size) {
	    		int c= (b+42)%256;
	    		if (c==0x00 || c==0x0A || c==0x0D || c==0x3D) {
	    			cb[cnt++] = (char) '=';
	    			cb[cnt++] = (char) ((c+64)%256);
	    			line+=2;
	    		} else if (line==0 && c == '.') {
	    			cb[cnt++] = (char) '.';
	    			cb[cnt++] = (char) '.';
	    		} else {
	    			cb[cnt++] = (char) c;
	    			line++;
	    		}
	    		if (line>=128) {
	    			cb[cnt++] = (char) '\r';
	    			cb[cnt++] = (char) '\n';
	    			line=0;
	    		}
	    	}
	    	cnt = createFooter(d,cb,cnt,useParts);
	    	bis.close();
	    	return String.valueOf(cb,0,cnt);
       		
    	} catch (IOException iex) {
    		return null;
    	} 
    	
    }
    
    
    
    public File decode() throws Exception {
        if (isStored()) {
            m_temp.close();
            return new File(System.getProperty("java.io.tmpdir"),((Descriptor) m_parts.get(0)).name);
        } else {
            return decode(new File(System.getProperty("java.io.tmpdir")));    
        }
        
    }
    
    public File decode(String path) throws Exception {
        File f=new File(path);
        return decode(f);
    }
    
    public File decode(File f) throws Exception {
        if (m_parts==null || m_parts.isEmpty()) {
            throw new Exception("Nothing to do");
        }
        if (f==null || !f.isDirectory()) {
            throw new Exception("Not a valid directory supplied");
        }
        Descriptor d=(Descriptor) m_parts.get(0);
        File out =new File(f,d.name);
        FileOutputStream fos=new FileOutputStream(out);
        TreeSet set=new TreeSet(m_parts);
        Iterator it = set.iterator();
        while (it.hasNext()) {
            d=(Descriptor) it.next();
            if (d.stored) {
                byte[] b= new byte [d.end-d.begin];
                m_temp.seek(d.begin);
                m_temp.read(b);
                fos.write(b);   
            } else {
                fos.write(d.data);
            }
        }
        if (m_temp!=null) {
            m_temp.close();
            new File(System.getProperty("java.io.tmpdir"),((Descriptor) m_parts.get(0)).name).delete();
        }
        fos.close();
        return out;
    }
    
    public byte[] decodeBytes() throws Exception {
    	
        TreeSet set=new TreeSet(m_parts);
    	Iterator it = set.iterator();
    	ByteArrayOutputStream baos=new ByteArrayOutputStream(((Descriptor) m_parts.get(0)).size);
        while (it.hasNext()) {
            Descriptor d=(Descriptor) it.next();
            //m_fileName=d.name;
        	baos.write(d.data);    
        }
        byte[] b= baos.toByteArray();
        baos.close();
        return b;
    }
    public boolean isComplete() {
        return true;
    }
    
    public boolean isStored() {
        return m_temp!=null;
    }
    
    private Descriptor _decode(char[] buf) throws Exception{
        Descriptor descriptor=new Descriptor();
        StringBuffer openString=null;
		//int cnt=0;
        int j=0;
        for (int i=0;i<buf.length;i++) {
            if (descriptor.finished) {
                break;
            }
            if (openString==null && buf[i]=='=') {
                switch(buf[++i]) {
                	case 'y':
                		openString=new StringBuffer();
                		break;
                	default:
                	    int o= buf[i]-106;
                        if (descriptor.end> 0 && descriptor.data==null) {
                            descriptor.data=new byte[descriptor.end-descriptor.begin];
                        }
                        descriptor.data[j++]=(byte) o;
               			
                }
            } else if (openString!=null) {
                if (buf[i]=='\n') {
                 	parseLine(descriptor,openString.toString());
                 	openString=null;
                } else {
                	openString.append(buf[i]);
                }
            } else if (descriptor.end>0){
                if (descriptor.data==null) {
                    descriptor.data=new byte[descriptor.end-descriptor.begin];
                }
                if (buf[i]=='\r' && buf[i+1]=='\n') {
                    if (buf.length>i+3 && buf[i+2]=='.' && buf[i+3]=='.') {
                        i+=2;
                    } else {
                        i++;
                    }
                } else {
                    int o= buf[i]-42;
               		if (j<descriptor.data.length) {
                        descriptor.data[j++]=(byte) o;
                    }
                }
            }
        }
        return descriptor;
    }

    private static void parseLine(Descriptor d, String s) {
        if (s.startsWith("begin")) {
            d.name=untilend(s,"name");
            d.line=intfrom(s,"line");
            d.size=intfrom(s,"size");
            d.total=intfrom(s,"total");
            d.end=d.size;
            d.part=intfrom(s,"part");
        } else if (s.startsWith("part")) {
            d.begin=intfrom(s,"begin")-1;
            d.end=intfrom(s,"end");
        } else if (s.startsWith("end")) {
            d.finished=true;
        }
    }
    
    private void store(Descriptor d) throws IOException {
        if (m_temp==null) {
            m_temp=new RandomAccessFile(new File(System.getProperty("java.io.tmpdir"),d.name),"rw");
        }
        m_temp.seek(d.begin);
        m_temp.write(d.data);
        d.data=null;
        d.stored=true;
    }
    
    private static String untilend(String s,String key) {
        int idx=s.indexOf(key);
        if (idx>0) {
            return s.substring(idx+key.length()+1).trim();
        }
        return null;
    }
    
    private static String from(String s,String key) {
        int idx=s.indexOf(key);
        if (idx>0) {
            int last=s.indexOf(' ',idx+key.length());
            if (last==-1) {
                last=s.length();
            }
            return s.substring(idx+key.length()+1,last).trim();
        } else {
            return null;
        }
    }
    
    private static int intfrom(String s,String key) {
        String l=from(s,key);
        if (l!=null && l.length()>0) {
            return Integer.parseInt(l);
        } else {
            return -1;
        }
    }
    public String getFileName() {
        return ((Descriptor) m_parts.get(0)).name;
    }
    
    private int createFooter(Descriptor d, char[] cb, int pos, boolean useParts) {
    	StringBuffer sb= new StringBuffer();
    	sb.append("\r\n=yend");
    	int size = d.size*d.part<d.total?d.size:d.total-d.size*(d.part-1);
    	sb.append(" size=").append(size);
    	if (useParts) {
    		sb.append(" part=").append(d.part);
    	}
    	sb.append("\r\n");
		if (sb.length()>cb.length+pos) cb = grow(cb,sb.length());
		sb.getChars(0, sb.length(), cb, pos);
		return pos+sb.length();
    }
    
    private int createHeader(Descriptor d, char[] cb, boolean useParts) {
    	StringBuffer sb= new StringBuffer();
    	sb.append("\r\n\r\n=ybegin");
    	if (useParts) {
    		sb.append(" part=").append(d.part);
    	}
    	sb.append(" line=").append(d.line).append(" size=").append(d.total).append(" name=").append(d.name).append("\r\n");
		if (sb.length()>cb.length) cb = grow(cb,sb.length());
		sb.getChars(0, sb.length(), cb, 0);
		return sb.length();
    }
    
    private int createPartHeader(Descriptor d, char[] cb, int pos) {
    	StringBuffer sb= new StringBuffer();
    	int size = d.size*d.part<d.total?d.size*d.part:d.total;
    	sb.append("=ypart begin=").append((d.part-1)*d.size+1).append(" end=").append(size).append("\r\n");
		if (sb.length()>cb.length) cb = grow(cb,sb.length());
		sb.getChars(0, sb.length(), cb, pos);
		return pos+sb.length();
    }
    
    private char[] grow(char[] cb, int add) {
    	char[] buf = new char[cb.length+add];
    	System.arraycopy(cb, 0, buf, 0, cb.length);
    	return buf;
    }
}
